from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RuntimeBusinessObject, WorkspaceObject, RevisionRule, Folder, Awb0Element, Awb0ProductContextInfo, ItemRevision
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06.OccurrenceManagement import InputContext5, ChildOccsLoadNxtCursor5
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05.OccurrenceManagement import DefaultName, ParentChildrenInfo, OccurrenceSortInput
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12.OccurrenceManagement import EffectivityRange, SourceContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12.OccurrenceManagement import OccurrenceCursor, OccurrenceInfo6, UserContextInfo, FocusOccurrenceInput
from tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10.OccurrenceManagement import InfoForElementCreation
from typing import Dict, List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import OccurrenceFilterInput, OccurrenceFilterOutput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AddObjectData2(TcBaseObj):
    """
    The structure contains object to be added as child of input parentElement or productContext.
    
    :var parentElement: The Awb0Element object under which objectsToBeAdded is added. Only parentElement or
    productContext may be set, not both.
    :var objectsToBeAdded: Business object to be added into product. This will be added under parentElement or
    productContext. It can be Item, ItemRevision, GenericDesignElement or Awb0Element.
    :var siblingElement: The newly added Awb0Element object should be added after this Awb0Element object.This is an
    optional parameter which is used for Requirement structures.
    :var inputCtxt: The input context has the information about filter, configuration, and page size.
    :var sortCriteria: Criteria to sort the occurrences based on internal property name and sorting order e.g. "ASC"
    and "DESC".
    :var addObjectIntent: Indicates user gesture. Possible value is 'DragAndDropIntent'
    :var fetchPagedOccurrences: If true then return the entire pageful along with created Awb0Element.
    :var requestPref: Map (string, string) of preference names and value pairs. Allowed preference names are:
    includeInterfaces , includeUnconfigured and . Allowed values are: True/False. Keys and values are case sensitive.
    :var numberOfElements: Indicates quantity of elements user wants to create.
    """
    parentElement: Awb0Element = None
    objectsToBeAdded: List[BusinessObject] = ()
    siblingElement: Awb0Element = None
    inputCtxt: InputContext5 = None
    sortCriteria: OccurrenceSortInput = None
    addObjectIntent: str = ''
    fetchPagedOccurrences: bool = False
    requestPref: RequestPreference9 = None
    numberOfElements: int = 0


@dataclass
class OccurrencesData2(TcBaseObj):
    """
    The OccurrencesData2 supplies the product and the configuration rules to configure the product and get a page of
    occurrences along with effective column configuration. Optionally, filters and sorting criteria may be provided to
    filter or sort the resulting occurrences.
    
    :var product: The top-level product. It may be an ItemRevision representing a product or a Mdl0odelElement such as
    a Cpd0CollaborativeDesign.
    :var parentElement: Parent occurrence for which child occurrences is to be retrieved.
    :var focusOccurrenceInput: Contains input information required to set the focusChildOccurrence.
    :var config: Specifies the complete set of configuration rules including RevisionRule, Effectivity and VariantRule
    to configure content.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var filter: A recipe to search and filter occurrences and sort the results.
    :var sortCriteria: Criteria to sort the occurrences based on intetnal property name and sorting order e.g. "ASC"
    and "DESC".
    :var requestPref: A map  (string, string) of preference names and value pairs. Allowed preference names are:
    "includeInterfaces" , and  "includeUnconfigured".   Allowed values are: True/False. Keys and values are case
    sensitive. When preference "includeInterface" with value true is specified the operation shall  return instances of
    Awb0Interface objects. When preference "includeUnconfigured" with value true is specified the operation shall
    return instances of unconfigured Awb0Element objects.
    """
    product: BusinessObject = None
    parentElement: str = ''
    focusOccurrenceInput: FocusOccurrenceInput = None
    config: OccurrenceConfigInput1 = None
    cursor: OccurrenceCursor = None
    filter: OccurrenceFilterInput = None
    sortCriteria: OccurrenceSortInput = None
    requestPref: RequestPreference2 = None


@dataclass
class OccurrencesResp2(TcBaseObj):
    """
    Contains the resulting occurrences found for product from which retrieval was requested. This data structure also
    returns a cursor to be sent back to the subsequent call to getOccurrences in order to rerieve more pages of results.
    
    :var rootProductContext: The Awb0ProductContextInfo for the root occurrence.
    :var parentProductContext: The Awb0ProductContextInfo for the parent occurrence.
    :var sourceContextToInfoMap: Map(WorkspaceObject, SourceContextInfo) containing pair of Source Context Object to
    its corresponding configuration information.
    :var parentChildrenInfos: List of parent and its children information. This will be populated only when
    loadTreeHierarchyThreshold requestPref is not empty.
    :var requestPref: A map (string, string) of preference names and value pairs. Allowed preference names are:
    "includeInterfaces" , and "includeUnconfigured". Allowed values are: True/False. Keys and values are case
    sensitive. When preference "includeInterface" with value true is specified the operation shall  return instances of
    Awb0Interface objects. When preference "includeUnconfigured" with value true is specified the operation shall
    return instances of unconfigured Awb0Element objects.
    :var serviceData: Contains the list of all BusinessObjects that make up the output, as well as any errors that
    might have occurred as part of the service invocation.
    :var parentOccurrence: The parent occurrence for list of childOccurrences.
    :var focusProductContext: The Awb0ProductContextInfo for the focus occurrence.
    :var focusChildOccurrence: The focus child occurrence for which sibling occurrences are desired.
    :var occurrences: List of resulting occurrences that exist in the configured structure. The configured occurrence
    object is of type Awb0Element
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var userWorkingContextInfo: User's working context information for opened content.
    :var filter: Updated search filter information.
    :var elementToPCIMap: Map containing pair of Root Occurrence (Awb0Element) to its corresponding
    Awb0ProductContextInfo object in Product Set container.
    """
    rootProductContext: Awb0ProductContextInfo = None
    parentProductContext: Awb0ProductContextInfo = None
    sourceContextToInfoMap: SourceContextToInfoMap2 = None
    parentChildrenInfos: List[ParentChildrenInfo] = ()
    requestPref: RequestPreference2 = None
    serviceData: ServiceData = None
    parentOccurrence: OccurrenceInfo6 = None
    focusProductContext: Awb0ProductContextInfo = None
    focusChildOccurrence: OccurrenceInfo6 = None
    occurrences: List[OccurrenceInfo6] = ()
    cursor: OccurrenceCursor = None
    userWorkingContextInfo: UserContextInfo = None
    filter: OccurrenceFilterOutput = None
    elementToPCIMap: ElementToProductContextInfoMap1 = None


@dataclass
class PackElementsData(TcBaseObj):
    """
    Contains a list of Awb0Element objects that are to be "pack" or "unpack", the sorting order criteria and the input
    configuration context.
    
    :var packMode: Indicates if input Awb0Element objects to be packed or unpacked. The valid values are:
    0 - packs the input Awb0Element objects
    1 - unpacks the input Awb0Element objects.
    :var elements: A list of Awb0Element objects that are to be "pack" or "unpack".
    :var sortCriteria: Criteria to sort the occurrences based on internal property name and sorting order e.g. "ASC"
    and  "DESC".
    :var inputCtxt: Input context has the information about filter, configuration, and page size.
    """
    packMode: int = 0
    elements: List[Awb0Element] = ()
    sortCriteria: OccurrenceSortInput = None
    inputCtxt: InputContext5 = None


@dataclass
class PackElementsResp(TcBaseObj):
    """
    A list of Awb0Element objects information on which "pack" or "unpack" action is performed. It also contains the
    information required to build the visible Awb0Element positions.
    
    :var elementsToBeSelected: A list of Awb0Element objects to be selected.
    :var parentChildrenInfos: A list of parent and its children information.
    :var serviceData: Service data containing the changed Awb0Element objects in updated objects list.
    """
    elementsToBeSelected: List[OccurrenceInfo6] = ()
    parentChildrenInfos: List[ParentChildrenInfo] = ()
    serviceData: ServiceData = None


@dataclass
class PagedOccurrencesInfo2(TcBaseObj):
    """
    List of paged child occurrences and the cursor representing the point to which results have been retrieved.
    
    :var childOccurrences: The list of child occurrences, also represent the page of occurrences.
    :var focusOccurrence: The focused occurrence is the occurrence which needs to be searched. The focusedOccurrence is
    the Awb0Element which user wants to find with-in given product.
    :var productContext: The Awb0ProductContextInfo to be associated with the parent that was requested for expand.
    There are scenarios where the given parent may be associated with a configuration context that is different from
    the root to configure it sub level contents. If NULL then configuration context is same as that of nearest parent
    occurrence having non-null value.
    :var cursor: Cursor that represents point to which results have been retrieved.
    :var startReached: If true, the childOccurrences is considered as the first page of occurrences. This value is used
    to identify the start location of occurrence page for two way scrolling functionality.
    :var endReached: If true, the childOccurrences is considered as the last page of occurrences. This value is used to
    identify the end location of occurrence page for two way scrolling functionality.
    """
    childOccurrences: List[OccurrenceInfo6] = ()
    focusOccurrence: Awb0Element = None
    productContext: Awb0ProductContextInfo = None
    cursor: ChildOccsLoadNxtCursor5 = None
    startReached: bool = False
    endReached: bool = False


@dataclass
class SelectedNewElementInfo2(TcBaseObj):
    """
    This structure contains the list of new Awb0Element objects added. List of paged child occurrences and the cursor
    representing the point to which results have been retrieved.
    
    :var newElements: This structure contains the list of new Awb0Element objects added. List of paged child
    occurrences and the cursor representing the point to which results have been retrieved.
    :var pagedOccurrencesInfo: List of paged child occurrences and the cursor representing the point to which results
    have been retrieved.
    """
    newElements: List[Awb0Element] = ()
    pagedOccurrencesInfo: PagedOccurrencesInfo2 = None


@dataclass
class UserContextState2(TcBaseObj):
    """
    This structure contains the current user's client state information for the opened object.
    
    :var openedObject: It can be the ItemRevision, Cpd0CollaborativeDesign or Awb0SavedBookmark object. The client
    state information like active tab, selected Awb0Element will be persisted for this object and the current user.
    :var sublocationAttributes: A map (string, list of strings) of client state attribute name and value pairs. At this
    time only one (key) attribute name is supported: "activeSublocation". Supported values are: (tab names like)
    "overview", "viewer". Keys and values are case in-sensitive. Content sublocation client state attributes that need
    to be persisted will be send to the client using this map.
    :var cloneContentSaveSpecifications: A map ( Awb0ProductContextInfo, list of 'cloneContentSaveAsIn') having
    information about various clone options like Clone, Reference and Ignore action per child Awb0Element per product.
    """
    openedObject: BusinessObject = None
    sublocationAttributes: SublocationAttributes3 = None
    cloneContentSaveSpecifications: CloneContentSaveSpecifications = None


@dataclass
class CloneContentData(TcBaseObj):
    """
    CloneContentData contains all the information needed to validate or duplicate the structure.
    
    :var productContextInfo: The Awb0ProductContextInfo object containing configuration information.
    :var defaultName: Contains the pattern to form the new IDs for the underlying objects (e.g ItemRevision) of the
    input Awb0Element objects.
    :var dataList: A list of Awb0Element objects and clone options.
    :var cloneFlags: A bit mask for the duplicate flags set.
                2 - Rename cad files.
                8 - Run in background mode.
    :var defaultFolder: The folder to add the duplicated structures to. 
    If NULL, the cloned assembly will not be added to a folder. User will have to search for it when needed.
    """
    productContextInfo: Awb0ProductContextInfo = None
    defaultName: DefaultName = None
    dataList: List[CloneContentSaveAsIn] = ()
    cloneFlags: int = 0
    defaultFolder: Folder = None


@dataclass
class CloneContentSaveAsIn(TcBaseObj):
    """
    Structure containing original Awb0Element objects and instructions which tells clone operation what to do with each
    component of the structure.
    
    :var element: Awb0Element object to be cloned.
    :var cloneOperationType: Indicates if the original object is to be:
    0 - Clone, The underlying object represented by Awb0Element to be cloned while instantiating in cloned structure.
    1 - Reference, The underlying object represented by Awb0Element to be referenced while instantiating in cloned
    structure.
    2 - Revise, The underlying object represented by Awb0Element to be revised while instantiating in cloned structure.
    3 - Replace, The underlying object represented by Awb0Element to be replaced by existing part while instantiating
    in cloned structure.
    5 - Ignore, The underlying object represented by Awb0Element should be ignored while instantiating in cloned
    structure.
    :var clonedObjectInfo: A map (string, string) of property names and its value pairs.  
    For example: 
    For the operation type Replace, the map will contain property "item_id" and "rev_id" as keys and their
    corresponding values as values.
    """
    element: Awb0Element = None
    cloneOperationType: int = 0
    clonedObjectInfo: PropertyValues = None


@dataclass
class AddObjectResp2(TcBaseObj):
    """
    The structure contains parent objects under which object was added. When serviceData containing Awb0Element as
    created objects.
    
    :var selectedNewElementInfo: List of new Awb0Element objects and paged occurrences for input parent under which add
    was performed.
    :var newElementInfos: List of NewElementInfo3 when the parent is instantiated more than once.
    :var reloadContent: if true, the content must be reloaded to see updated information.
    :var serviceData: The service data containing created Awb0Element in the created list and partial error if any.
    """
    selectedNewElementInfo: SelectedNewElementInfo2 = None
    newElementInfos: List[NewElementInfo3] = ()
    reloadContent: bool = False
    serviceData: ServiceData = None


@dataclass
class InfoForAddElemData2(TcBaseObj):
    """
    This structure contains information to identify type that can be added as child of input parentElement.
    
    :var parentElement: Awb0Element under which another Awb0Element to be added as child. This is specified if
    Awb0Element needs to be added under parent context. This is optional parameter.
    :var occurrenceTypeName: Internal Name of the occurrence type. This is specified to get the allowed related object
    type names for the given occurrence type name. This is optional parameter.
    """
    parentElement: Awb0Element = None
    occurrenceTypeName: str = ''


@dataclass
class InfoForAddElemResp2(TcBaseObj):
    """
    This structure contains allowable child type name, mapped Awb0Element type name and type name to be searched
    through Full Text Search and a flag indicating if the object type is of occurrence type.
    
    :var preferredTypeInfo: Preferred type information to create Awb0Element.
    :var allowedTypeInfos: List of all allowed types to create and add Awb0Element as child.
    :var preferredExists: true if preferred type is found for input Awb0Element, false otherwise.
    :var serviceData: The service data containing partial error if any.
    :var isOccurrence: If true, if allowedTypeInfos contains information for revisable occurrence; otherwise, false.
    """
    preferredTypeInfo: InfoForElementCreation = None
    allowedTypeInfos: List[InfoForElementCreation] = ()
    preferredExists: bool = False
    serviceData: ServiceData = None
    isOccurrence: bool = False


@dataclass
class NewElementInfo3(TcBaseObj):
    """
    This structure contains parent Awb0Element under which new Awb0Element objects was created.
    
    :var parentElement: Parent under which objects was added.
    :var newElements: List of new OccurrenceInfos created for the parent.
    :var newElementToPositionMap: Position of newly created Awb0Element within its parent.
    """
    parentElement: Awb0Element = None
    newElements: List[OccurrenceInfo6] = ()
    newElementToPositionMap: NewElementToPositionMap2 = None


@dataclass
class OccurrenceConfigInput1(TcBaseObj):
    """
    OcurrenceConfigInput1 specifies the complete set of configuration rules including RevisionRule, Effectivity
    with/without intent values, and VariantRule to configure content. When Awb0ProductContextInfo is specified all
    other configuration parameters are ignored.
    
    :var productContext: Awb0ProductContextInfo containing configuration information.
    :var revisionRule: RevisionRule to configure the product.
    :var sourceContext: Source Context (Fnd0RecipeContainer or Mdl0BaselineRevision) associated with another
    Fnd0RecipeContainer
    :var changeContext: Runtime Business object fnd0ChangeContext that holds the Change Context to be set and
    corresponding Change Configuration Mode.
    :var effectivityDate: Specify the date effectivity to configure the product.
    :var effectivityGroups: List of Fnd0EffGrpRevision objects to configure the product.
    :var unitNo: Unit number to use in order to configure the product.
    :var endItem: Specify the end Item which governs the unit effectivity.
    :var variantRules: List of VariantRule(s) or StoredOptionSet(s) to configure the product.
    :var svrOwningProduct: This indicates the product(ItemRevision or CollaborativeDesign) associated with saved
    VariantRule.
    :var effectivityRanges: List of effectivity data. This data may or may not have effectivity intent values
    associated with the effectivity.
    :var occurrenceScheme: Occurrence scheme associated with product e.g Cpd0CollaborativeDesign.
    """
    productContext: Awb0ProductContextInfo = None
    revisionRule: RevisionRule = None
    sourceContext: WorkspaceObject = None
    changeContext: RuntimeBusinessObject = None
    effectivityDate: datetime = None
    effectivityGroups: List[ItemRevision] = ()
    unitNo: int = 0
    endItem: BusinessObject = None
    variantRules: List[WorkspaceObject] = ()
    svrOwningProduct: BusinessObject = None
    effectivityRanges: List[EffectivityRange] = ()
    occurrenceScheme: BusinessObject = None


"""
A map ( Awb0ProductContextInfo, list of 'cloneContentSaveAsIn') having information about various clone options like Clone, Reference and Ignore action per child Awb0Element per product.
"""
CloneContentSaveSpecifications = Dict[Awb0ProductContextInfo, List[CloneContentSaveAsIn]]


"""
A map of property names and its value pairs (string, string).
"""
PropertyValues = Dict[str, str]


"""
Description:
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive.
"""
RequestPreference2 = Dict[str, List[str]]


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive. The allowed preference names are includeInterfaces. The allowed values are True/False.
"""
RequestPreference9 = Dict[str, List[str]]


"""
Map(WorkspaceObject, SourceContextInfo) containing pair of Source Context Object to its corresponding configuration information
"""
SourceContextToInfoMap2 = Dict[WorkspaceObject, SourceContextInfo]


"""
Map containing Awb0Element to its respective Awb0ProductContextInfo object.
"""
ElementToProductContextInfoMap1 = Dict[Awb0Element, Awb0ProductContextInfo]


"""
A map (string, list of string) of client state attribute name and value pairs. At this time only one (key) attribute name is supported: "activeSublocation". Supported values are: (tab names like) "overview", "viewer" . Keys and values are case in-sensitive.
"""
SublocationAttributes3 = Dict[str, List[str]]


"""
Map containing newly created Awb0Element to its position within parent.
"""
NewElementToPositionMap2 = Dict[Awb0Element, int]



from __future__ import annotations

from tcsoa.gen.Internal.GlobalMultiSite._2008_06.Synchronizer import TransferFormula, GetExportedObjectsResponse
from typing import List
from tcsoa.gen.Internal.GlobalMultiSite._2007_06.Synchronizer import SyncResponse, ObjectsByClass
from tcsoa.base import TcService


class SynchronizerService(TcService):

    @classmethod
    def getCandidatesForClasseswOpts(cls, targetSite: int, classList: List[str], transferFormula: TransferFormula) -> SyncResponse:
        """
        This is an internal operation which returns a list of candidates that need to be synchronized based on the
        input set of classes and target site. Candidates for the synchronization will be identified based on the
        ImanExportRecord availability for the objects of input class or transfer formula passed to this operation.
        
        Use cases:
        
        """
        return cls.execute_soa_method(
            method_name='getCandidatesForClasseswOpts',
            library='Internal-GlobalMultiSite',
            service_date='2008_06',
            service_name='Synchronizer',
            params={'targetSite': targetSite, 'classList': classList, 'transferFormula': transferFormula},
            response_cls=SyncResponse,
        )

    @classmethod
    def getCandidatesForObjectswOpts(cls, targetSite: int, objectList: List[ObjectsByClass], transferFormula: TransferFormula) -> SyncResponse:
        """
        This is an internal operation. It returns a list of candidates for synchronization. User can specify particular
        objects and class to which it belongs. If objects are specified then only those objects are checked for
        modification. If objects are not specified, all the objects belonging to that class are checked for
        synchronization. Objects that are replicated at target site are taken into account. If invalid class is
        provided, it gets ignored.
        
        Use cases:
        Find modified objects since last export for a particular class and set of objects belonging to that class.
        """
        return cls.execute_soa_method(
            method_name='getCandidatesForObjectswOpts',
            library='Internal-GlobalMultiSite',
            service_date='2008_06',
            service_name='Synchronizer',
            params={'targetSite': targetSite, 'objectList': objectList, 'transferFormula': transferFormula},
            response_cls=SyncResponse,
        )

    @classmethod
    def getExportedObjects(cls, targetSiteId: int, classList: List[str]) -> GetExportedObjectsResponse:
        """
        This operation returns a list of objects that were exported to the target site for a given list of classes.
        
        Use cases:
        
        """
        return cls.execute_soa_method(
            method_name='getExportedObjects',
            library='Internal-GlobalMultiSite',
            service_date='2008_06',
            service_name='Synchronizer',
            params={'targetSiteId': targetSiteId, 'classList': classList},
            response_cls=GetExportedObjectsResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.BusinessObjects import POM_imc
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SiteInfo(TcBaseObj):
    """
    This object represents a POM_imc business object along with all the information that belongs to that object.
    
    :var siteID: The unique numeric identification number for the site, cannot be 0 but can be negative.
    :var siteName: The name of the site, accepts all ASCII characters but cannot be an empty string.
    :var additionalPropMap: A map (string, string) for the additional properties to create or modify on the POM_imc
    business object. The map has the following Key/Value pairs representing the property name and property's value
    respectively.
    """
    siteID: int = 0
    siteName: str = ''
    additionalPropMap: AdditionalProperty = None


@dataclass
class CreateOrUpdateSitesResponse(TcBaseObj):
    """
    Response structure of createOrUpdateSite operation.
    
    :var outputs: The information that is given back upon a successful creation or update of a POM_imc business object.
    Information includes the object's name, object's ID, and any additional properties the object has.
    :var serviceData: The service data with partial errors if any.
    """
    outputs: List[CreateSitesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateSitesOutput(TcBaseObj):
    """
    The output from creating or updating a POM_imc business object, including the object itself.
    
    :var siteInfo: Information about the POM_imc business object that was updated or created.
    :var pomIMC: The business object POM_imc retrieved from the database.
    """
    siteInfo: SiteInfo = None
    pomIMC: POM_imc = None


"""
A key / value pair for additional properties of the site.

NOTE: This is used for bool types and string types. For bool types we are assuming that the valid values are:

true: "true" (any form of capitalization)
false: "false" (any form of capitalization)
"""
AdditionalProperty = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TransferOptionSet
from tcsoa.gen.GlobalMultiSite._2007_06.ImportExport import GetAllTransferOptionSetsResponse, CreateOrUpdatePropertySetRuleResponse, GetPropertySetsResponse, CreateOrUpdatePropertySetInputData, GetTransferModesResponse, CreateOrUpdateTransferModeResponse, CreateOrUpdateClosureRuleInputData, CreateOrUpdateTransferModeInputData, CreateOrUpdateFilterRuleResponse, CreateOrUpdateActionRuleInputData, CreateOrUpdateActionRuleResponse, GetAvailableTransferOptionSetsInputData, GetAvailableTransferOptionSetsResponse, GetActionRulesResponse, CreateOrUpdateClosureRuleResponse, GetPLMXMLRuleInputData, NamesAndValue, CreateOrUpdateTransferOptionSetInputData, RequestImportFromOfflinePackageResponse, CreateOrUpdateTransferOptionSetResponse, GetFilterRulesResponse, CreateOrUpdateFilterRuleInputData, GetClosureRulesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def getPropertySets(cls, inputs: GetPLMXMLRuleInputData) -> GetPropertySetsResponse:
        """
        This operation return a set of property set objects depending upon input query parameters.
        """
        return cls.execute_soa_method(
            method_name='getPropertySets',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=GetPropertySetsResponse,
        )

    @classmethod
    def getTransferModes(cls, inputs: GetPLMXMLRuleInputData) -> GetTransferModesResponse:
        """
        This operation returns a set of transfer mode objects depending upon input query parameters.
        """
        return cls.execute_soa_method(
            method_name='getTransferModes',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=GetTransferModesResponse,
        )

    @classmethod
    def requestImportFromOfflinePackage(cls, fmsTicket: str, optionSetTag: TransferOptionSet, optionNamesAndValues: List[NamesAndValue], sessionOptionAndValues: List[NamesAndValue]) -> RequestImportFromOfflinePackageResponse:
        """
        This operation imports the contents of the briefcase container into database by placing a request to the Global
        Services (GS) components. This operation is very much similar to importObjectsFromOfflinePackage with the
        exception that this operation is used in GS enabled environment whereas importObjectsFromOfflinePackage
        operation is used in Non GS environment. A packed briefcase contains a TC XML file which holds a serious of
        Teamcenter objects and related physical dataset files. After import, those objects will be replica in the
        importing site.
        
        Use cases:
        In data exchange, user may transfer a briefcase file from the source site to a remote site. In the importing
        site, user can use this operation to import the briefcase file into the Teamcenter. After import, the objects
        held in the TC XML file will be created or updated if they have been imported before, physical dataset files
        will uploaded and attached to the related datasets.
        The SOA needs the GS (Global Service) been configured for the importing site.
        """
        return cls.execute_soa_method(
            method_name='requestImportFromOfflinePackage',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'fmsTicket': fmsTicket, 'optionSetTag': optionSetTag, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionAndValues': sessionOptionAndValues},
            response_cls=RequestImportFromOfflinePackageResponse,
        )

    @classmethod
    def createOrUpdateActionRules(cls, inputs: List[CreateOrUpdateActionRuleInputData]) -> CreateOrUpdateActionRuleResponse:
        """
        Creates or updates an action rule based on input parameters. Action rule in the PLM XML framework is used to
        invoke additional actions before/during/after import/export. For more information on action rules, please refer
        to PLM XML Import Export Administration Guide.
        
        Use cases:
        Use Case 1: Modify an Action Rule
        The following types of modifications can be done on existing action rule using 'createOrUpdateActionRules'
        operation:
        - Change the rule description.
        - Change the action handler. This means that we can change the action rule's clause to invoke a different
        action than what was initially assigned.
        - Change the action location. This means we can change action location from pre-action to post-action etc.
        - Change the schema format. This means we can change the action rule from PLM XML schema based one to TC XML
        schema.
        - Change data transfer direction scope. This means we can change the direction from export to import and
        vice-versa.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateActionRules',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=CreateOrUpdateActionRuleResponse,
        )

    @classmethod
    def createOrUpdateClosureRules(cls, inputs: List[CreateOrUpdateClosureRuleInputData]) -> CreateOrUpdateClosureRuleResponse:
        """
        Creates or updates a closure rule based on input parameters. Closure rule specify how the data structure is
        traversed by specifying which relationships are of interest and what should be done when these relationships
        are encountered. For more information, please refer to PLM XML Import Export Administration Guide.
        
        Use cases:
        Use Case 1: Modify a Closure Rule
        The following types of modifications can be done on existing closure rule using 'createOrUpdateClosureRules'
        operation:
        - Change the closure rule description.
        - Change schema format. This means we can change the closure rule from PLM XML schema based one to TC XML
        schema.
        - Change transfer direction. This means we can change the direction from export to import and vice-versa.
        - Change clause contents, depth and comments for each clause. You can change detailed clauses in this closure
        rule. For more information to how to write clauses, please refer to PLM XML Import Export Administration Guide.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateClosureRules',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=CreateOrUpdateClosureRuleResponse,
        )

    @classmethod
    def createOrUpdateFilterRules(cls, inputs: List[CreateOrUpdateFilterRuleInputData]) -> CreateOrUpdateFilterRuleResponse:
        """
        Creates or updates a filter rule based on input parameters. Filter rules allow a finer level of control over
        the data that gets translated along with the primary objects by specifying that a user-written function is
        called to determine the operation applied against a given object. For more information, please refer to PLM XML
        Import Export Administration Guide.
        
        Use cases:
        Use Case 1: Modify a Filter Rule
        The following types of modifications can be done on existing filter rule using 'createOrUpdateFilterRules'
        operation:
        - Change the filter rule description.
        - Change clauses. For more information about how to write clause, please refer to PLM XML Import Export
        Administration Guide.
        - Change schema format. This means we can change the filter rule from PLM XML schema based one to TC XML schema.
        - Change data transfer direction scope. This means we can change the direction from export to import and
        vice-versa.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateFilterRules',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=CreateOrUpdateFilterRuleResponse,
        )

    @classmethod
    def createOrUpdatePropertySets(cls, inputs: List[CreateOrUpdatePropertySetInputData]) -> CreateOrUpdatePropertySetRuleResponse:
        """
        Creates or updates a property set based on input parameters. Property sets provide a non-programmatic way to
        control what is placed in the UserData element. For more information, please refer to PLM XML Import Export
        Administration Guide.
        
        Use cases:
        Use Case 1: Modify a Property Set
        The following types of modifications can be done on existing property set using 'createOrUpdatePropertySets'
        operation:
        - Change the property set description.
        - Change data transfer direction scope. This means we can change the direction from export to import and
        vice-versa.
        - Change clauses. For more information about how to write clause, please refer to PLM XML Import Export
        Administration Guide.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdatePropertySets',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=CreateOrUpdatePropertySetRuleResponse,
        )

    @classmethod
    def createOrUpdateTransferModes(cls, inputs: List[CreateOrUpdateTransferModeInputData]) -> CreateOrUpdateTransferModeResponse:
        """
        Creates or updates a transfer mode based on input parameters. Transfer modes are created in the PLMXML
        application. Transfer modes define how to import/export data between PLMXML file and sites. For more
        information, please refer to PLM XML Import Export Administration Guide.
        
        Use cases:
        Use Case 1: Modify a Transfer Mode
        The following types of modifications can be done on existing transfer mode using 'createOrUpdateTransferModes'
        operation. 
        - Change the transfer mode description
        - Change context string. Context string is used to map the transfer mode object to a customized processor for
        the given object type. For more information, please refer to PLM XML Import Export Administration Guide.
        - Change data transfer direction scope. This means we can change the direction from export to import and
        vice-versa.
        - Change schema format. This means we can change the closure rule from PLM XML schema based one to TC XML
        schema.
        - Change Incremental setting.  This option allows updates to existing data during PLM XML import. For example,
        if an item being imported from an .xml file already exists in the database and "support incremental" is
        selected, the PLM XML import updates the item. If "support incremental" is not selected, the updates from the
        .xml file are ignored.
        - Change closure rule, filter rule, property set, revision rule and action rule which are used by this transfer
        mode.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateTransferModes',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=CreateOrUpdateTransferModeResponse,
        )

    @classmethod
    def createOrUpdateTransferOptionSets(cls, inputs: List[CreateOrUpdateTransferOptionSetInputData]) -> CreateOrUpdateTransferOptionSetResponse:
        """
        Creates or update a list of transfer option sets based on the input properties structure. The transfer option
        set contains a set of variables which will control the export/import behavior. For more information, please
        refer to PLM XML Import Export Administration Guide.
        
        Use cases:
        Use Case 1: Modify a Transfer Option Set
        The following types of modifications can be done on existing transfer option set using
        createOrUpdateTransferOptionSets operation: 
        - Change the transfer option set description
        - Change referenced site id. It shows whether the transfer option set is for a remote site, thus an import. If
        so, its remote site ID is included. 
        - Change the attached transfer mode id.
        - Change the detail options for the transfer option set. For more information to the options, please refer to
        PLM XML Import Export Administration Guide.
        
        
        
        Exceptions:
        >203406    If the operation fails to create transfer option set.
        203407    If the operation fails to modify transfer option set.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateTransferOptionSets',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=CreateOrUpdateTransferOptionSetResponse,
        )

    @classmethod
    def getActionRules(cls, inputs: GetPLMXMLRuleInputData) -> GetActionRulesResponse:
        """
        This operation return a set of action rule objects depending upon input query parameters.
        
        Exceptions:
        >203419    If schema input is incorrect.
        203420    If the scope input is incorrect.
        203424    If the query cannot find action rules.
        203414    If the query fails to execute to while fetching the action rules from database.
        """
        return cls.execute_soa_method(
            method_name='getActionRules',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=GetActionRulesResponse,
        )

    @classmethod
    def getAllTransferOptionSets(cls) -> GetAllTransferOptionSetsResponse:
        """
        This operation return a set of transfer option set objects that were created with scope - public.
        
        Exceptions:
        >Other exception throwed by called API.
        """
        return cls.execute_soa_method(
            method_name='getAllTransferOptionSets',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={},
            response_cls=GetAllTransferOptionSetsResponse,
        )

    @classmethod
    def getAvailableTransferOptionSets(cls, inputs: GetAvailableTransferOptionSetsInputData) -> GetAvailableTransferOptionSetsResponse:
        """
        This operation return a set of transfer option set object depending upon input query parameters.
        """
        return cls.execute_soa_method(
            method_name='getAvailableTransferOptionSets',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=GetAvailableTransferOptionSetsResponse,
        )

    @classmethod
    def getClosureRules(cls, inputs: GetPLMXMLRuleInputData) -> GetClosureRulesResponse:
        """
        This operation return a set of closure rule objects depending upon input query parameters.
        """
        return cls.execute_soa_method(
            method_name='getClosureRules',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=GetClosureRulesResponse,
        )

    @classmethod
    def getFilterRules(cls, inputs: GetPLMXMLRuleInputData) -> GetFilterRulesResponse:
        """
        This operation return a set of filter rule objects depending upon input query parameters.
        """
        return cls.execute_soa_method(
            method_name='getFilterRules',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=GetFilterRulesResponse,
        )


class SiteReservationService(TcService):

    @classmethod
    def siteCheckIn(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation is used in offline GMS. It is used to check in objects that were checked out to another site and
        removes the reservation objects. If errors occur, they are returned in the ServiceData structure.
        
        Use cases:
        User can pass a list of objects which have been in site checked out status to do site check in. A series of
        related objects will be site checked in along with the input objects Item, ItemRevision or Dataset:
        - Item        Related Master Form(s)
        - ItemRevision    Related Master Form(s)
        - Dataset        All namedReference objects which are WorkspaceObject
        
        """
        return cls.execute_soa_method(
            method_name='siteCheckIn',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='SiteReservation',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def siteCheckOut(cls, objects: List[BusinessObject], siteId: int, comment: str, changeId: str) -> ServiceData:
        """
        This operation is used in offline GMS. It is used to checkout objects to another site, so that after the
        objects are imported into that site, they are modifiable. If errors occur, they are returned in the ServiceData
        structure.
        
        Use cases:
        User can pass a list of objects (only these six types and their sub types are supported: Item, ItemRevision,
        Form, Dataset, BOMView, BOMViewRevision) to do site check out. A series of related objects will be site checked
        out along with the input objects Item, ItemRevision or Dataset:
        - Item        Related Master Form(s)
        - ItemRevision    Related Master Form(s)
        - Dataset        All namedReference objects which are WorkspaceObject
        
        
        
        Exceptions:
        >.
        """
        return cls.execute_soa_method(
            method_name='siteCheckOut',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='SiteReservation',
            params={'objects': objects, 'siteId': siteId, 'comment': comment, 'changeId': changeId},
            response_cls=ServiceData,
        )

    @classmethod
    def cancelSiteCheckOut(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation is used in offline GMS. It is used to cancel site check out objects which has been site checked
        out to another site and removes the reservation objects. If errors occur, they are returned in the ServiceData
        structure.
        
        Use cases:
        User can pass a list of objects which have been in site checked out status to do cancel site check in.
        """
        return cls.execute_soa_method(
            method_name='cancelSiteCheckOut',
            library='GlobalMultiSite',
            service_date='2007_06',
            service_name='SiteReservation',
            params={'objects': objects},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FavoritesContainer(TcBaseObj):
    """
    A Favorites object that contains one or more subordinate favorite containers and/or Teamcenter objects supporting a
    hierarchical favorites structure.
    
    :var clientId: The identifier that relates a set of favorites container information to reported errors.
    :var id: The container identifier that is unique for all favorites. The identifier format is four numeric
    characters with leading zeros.
    :var type: The container type.
    :var displayName: The display name used for the favorites container.
    :var parentId: The parent container identifier.  An identifier of 0000 indicates the root container.
    """
    clientId: str = ''
    id: str = ''
    type: str = ''
    displayName: str = ''
    parentId: str = ''


@dataclass
class FavoritesInfo(TcBaseObj):
    """
    Input information for setting the favorites for the session user.
    
    :var curFavorites: The client copy of the current favorites containers and objects. If current containers and
    objects are specified, they must match the current saved favorites exactly for the specified new favorites to be
    saved.  If 'curFavorites' does not match the current saved favorites in Teamcenter, 'newFavorites' will not be
    saved.  This provides the client the ability to protect against simultaneous updates by two clients.
    :var newFavorites: The new favorites containers and objects to be saved.  The containers and objects specified in
    'newFavorites' make up the entire list of favorites.  If no new containers and no new objects are specified, the
    saved favorites for the session user will be cleared.
    """
    curFavorites: FavoritesList = None
    newFavorites: FavoritesList = None


@dataclass
class FavoritesList(TcBaseObj):
    """
    A hierarchical favorites tree structure list that contains all the favorites containers and favorites objects for
    the current session user. The favorites section can be populated only if 'FavoritesSection' is passed as a key in
    the input map to this API.
    
    :var containers: List of favorite containers for the current session user.
    :var objects: List of favorite objects for the current session user.
    """
    containers: List[FavoritesContainer] = ()
    objects: List[FavoritesObject] = ()


@dataclass
class FavoritesObject(TcBaseObj):
    """
    A Teamcenter favorites object.
    
    :var clientId: The identifier that relates a set of favorites object information to reported errors.
    :var objectTag: The favorite object.
    :var displayName: The display name used for the favorites object.
    :var parentId: The parent container identifier.  An identifier of 0000 indicates the root container.
    """
    clientId: str = ''
    objectTag: BusinessObject = None
    displayName: str = ''
    parentId: str = ''


@dataclass
class FavoritesResponse(TcBaseObj):
    """
    The set of Favorites containers and Favorites objects that define a hierarchical Favorites structure for the
    current session user.
    
    :var output: List of favorite containers and favorite objects for the current session user.
    :var serviceData: The service data. This operation will populate the Service Data plain objects list with the
    Favorite containers and objects.
    """
    output: FavoritesList = None
    serviceData: ServiceData = None


@dataclass
class ConnectResponse(TcBaseObj):
    """
    Indicates number of licenses avaliable and any failure
    
    :var outputVal: The number of available licenses for the specified featureKey parameter.
    :var serviceData: The ServiceData structure contains these potential error numbers:
          214401: The initialization of the license module has failed.
          214402: Detaching the requested license has failed.
          214403: The deallocation of the license feature key has failed.
          214404: Checking of the license feature key has failed.
          214405: The allocation of the license feature key has failed.
          214406: The module is not purchased.
          214407: The passed in action is invalid.
    """
    outputVal: int = 0
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Reports._2007_06.CubeReports import ConstructReportURLResponse, TcRAReportsCriteria
from tcsoa.base import TcService


class CubeReportsService(TcService):

    @classmethod
    def constructReportURL(cls, inputCriteria: TcRAReportsCriteria) -> ConstructReportURLResponse:
        """
        Constructs the servlet URL required by clients to process the TcRA report definition operation (retrieve, view,
        edit, delete or set permission) specified. The report definition ID, context objects, message name is required
        to get this URL.
        
        Use cases:
        User retrieves/views/edits/deletes/sets permission for one TcRA report, this operation will generate the
        corresponding URL for the TcRA operation.
        """
        return cls.execute_soa_method(
            method_name='constructReportURL',
            library='Reports',
            service_date='2007_06',
            service_name='CubeReports',
            params={'inputCriteria': inputCriteria},
            response_cls=ConstructReportURLResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.base import TcService
from tcsoa.gen.Administration._2015_03.UserManagement import UserProfileProperties, UserProfilePropertiesResponse


class UserManagementService(TcService):

    @classmethod
    def setUserProfileProperties(cls, userProfileInputs: List[UserProfileProperties]) -> UserProfilePropertiesResponse:
        """
        This operation sets the given properties on the Fnd0CustomUserProfile of the specified User. A new
        Fnd0CustomUserProfile object will be created if it does not already exist and the fnd0custom_user_profile
        property of the User will be set to the newly created Fnd0CustomUserProfile object.
        
        Use cases:
        Use Case 1: Set the properties for a User who does not have a Custom User Profile object.
        Use Case 2: Set the properties for a User who has a Custom User Profile object.
        """
        return cls.execute_soa_method(
            method_name='setUserProfileProperties',
            library='Administration',
            service_date='2015_03',
            service_name='UserManagement',
            params={'userProfileInputs': userProfileInputs},
            response_cls=UserProfilePropertiesResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetODSVersionResponse(TcBaseObj):
    """
    'GetODSVersionResponse'  structure contains ODS version information and 'Servicedata'.
    
    :var serverReleaseVersion: Major release version of Teamcenter server hosting the ODS.
    :var serverMinorReleaseVersion: Minor release version of Teamcenter server hosting the ODS.
    :var serverServiceReleaseVersion: Service release version of Teamcenter server hosting the ODS.
    :var serverMaintPackVersion: Maintenance pack version of the server hosting the ODS.
    :var serverOdsMajorVersion: ODS server major version.
    :var serverOdsMinorVersion: ODS server minor version.
    :var clientRejected: Access to the requesting TcEnt client rejected/ accepted.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    """
    serverReleaseVersion: int = 0
    serverMinorReleaseVersion: int = 0
    serverServiceReleaseVersion: int = 0
    serverMaintPackVersion: int = 0
    serverOdsMajorVersion: int = 0
    serverOdsMinorVersion: int = 0
    clientRejected: bool = False
    serviceData: ServiceData = None


@dataclass
class ItemIdRecordReturnStruct(TcBaseObj):
    """
    'ItemIdRecordReturnStruct' structure contains Item registration information.
    
    :var siteId: Owning siteid of input item.
    :var isRegistered: Flag for item registration status, whether input itemid is registered or not in the ItemId
    registry.
    """
    siteId: int = 0
    isRegistered: bool = False


@dataclass
class ItemIdRecordsProperties(TcBaseObj):
    """
    'ItemIdRecordsProperties' structure which contains item records to be registered in Item registry.
    
    :var itemId: The ID of the Item to be registered.
    :var context: Not being used.
    :var registrationSite: Siteid of the owning site of given itemId.
    """
    itemId: str = ''
    context: str = ''
    registrationSite: int = 0


@dataclass
class LocatePublishedObjectResponse(TcBaseObj):
    """
    'LocatePublishedObjectResponse' structure contains locating information for the input object and 'Servicedata'.
    
    :var owningSiteId: List of owning siteids for all the input Published Objects tag  strings.
    
    :var serviceData: The Service Data with failure message if any.
    """
    owningSiteId: List[int] = ()
    serviceData: ServiceData = None


@dataclass
class PublicationRecordProperties(TcBaseObj):
    """
    'PublicationRecordProperties' structure which contains attributes of Published object which is published to ODS.
    
    :var owningSiteId: Siteid of owning site of published object.
    :var objectTag: Tag of the published object.
    :var creationDate: Date of creation of published Object.
    :var pubDate: Date of publication of published Object.
    :var releaseStatus: Relase status if any of published Object is released.
    :var customAttributes: It is a string Map of custom attributes of published Object and ODS custom attribute.  This
    is used in case of extensible ODS feature.
    :var objectId: ID of the published object.
    :var revisionId: Revision id of the published object.
    :var objectName: Object name of published Object.
    :var objectDesc: Object description of published Object
    :var objectClass: Object class name of published Object.
    :var objectType: Object tyoe name of published Object.
    :var owningUserId: Owning user id of published Object.
    :var owningGroupName: Owning group name of published Object.
    """
    owningSiteId: int = 0
    objectTag: str = ''
    creationDate: datetime = None
    pubDate: datetime = None
    releaseStatus: List[str] = ()
    customAttributes: MapData = None
    objectId: str = ''
    revisionId: str = ''
    objectName: str = ''
    objectDesc: str = ''
    objectClass: str = ''
    objectType: str = ''
    owningUserId: str = ''
    owningGroupName: str = ''


@dataclass
class QueryItemIdRecordResponse(TcBaseObj):
    """
    'QueryItemIdRecordResponse' structure contains output query results in 'ItemIdRecordReturnStruct' object and
    'ServiceData'
    
    :var queryResults: List of 'ItemIdRecordReturnStruct' object which contains owning site id and flag for
    registration for each input itemid.
    :var serviceData: The Service Data with failure message if any.
    """
    queryResults: List[ItemIdRecordReturnStruct] = ()
    serviceData: ServiceData = None


@dataclass
class QueryPublicationRecordsResponse(TcBaseObj):
    """
    'QueryPublicationRecordsResponse' structure contains information returned from the query and 'ServiceData'.
    
    :var publishedObjTagAsString: List of published objects tag strings which returned as remote search results.
    :var serviceData: The Service Data with failure message if any.
    """
    publishedObjTagAsString: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ClientInfoProperties(TcBaseObj):
    """
    'ClientInfoProperties' structure contains requesting ODS client information.
    
    
    :var clientSiteId: Requesting client siteid.
    :var targetServerSiteId: Target server siteid.
    :var userName: Logged in username of requesting client.
    :var osUserName: OS logged in username of requesting client node.
    :var groupName: Group name of userid of requesting client.
    
    :var roleName: Role name of userid of requesting client.
    
    :var customAttributeNames: String map of custom attributes of published Object and ODS custom attribute. This is
    used in case of extensible ODS feature.
    :var clientNode: Node name of requesting client.
    :var clientReleaseVersion: Teamcenter release version of requesting client.
    :var clientMinorReleaseVersion: Minor release version of requesting client.
    :var clientServiceReleaseVersion: Teamcenter  release version of requesting client server.
    :var clientMaintPackVersion: Teamcenter  maintaince pack version of requesting client.
    :var clientOdsMajorVersion: Client ODS major version.
    :var clientOdsMinorVersion: Client ODS minor version.
    :var userId: Logged in userid of requesting client.
    """
    clientSiteId: int = 0
    targetServerSiteId: int = 0
    userName: str = ''
    osUserName: str = ''
    groupName: str = ''
    roleName: str = ''
    customAttributeNames: MapData = None
    clientNode: str = ''
    clientReleaseVersion: int = 0
    clientMinorReleaseVersion: int = 0
    clientServiceReleaseVersion: int = 0
    clientMaintPackVersion: int = 0
    clientOdsMajorVersion: int = 0
    clientOdsMinorVersion: int = 0
    userId: str = ''


@dataclass
class DescribePublicationRecordsResponse(TcBaseObj):
    """
    'DescribePublicationRecordsResponse' structure contains list of all PublicationRecord properties and Servicedata.
    
    :var records: List of 'PublicationRecordProperties' object for all the input PublishedObjects tag strings.
    :var serviceData: The 'ServiceData' with failure message if any.
    """
    records: List[PublicationRecordProperties] = ()
    serviceData: ServiceData = None


@dataclass
class DistributedAppProperties(TcBaseObj):
    """
    'DistributedAppProperties' structure contains required information to call the DIST ITK.
    
    :var clientReleaseVersion: Requesting client release version.
    :var clientIrmNumber: Teamcenter version release, phase number.
    :var appName: Application name for corressponding dist ITK.
    :var opCode: Method opcode to perform operation.
    :var appInputStringVal: List of input arguments which are in encoded form given to particular DIST itk call.
    """
    clientReleaseVersion: int = 0
    clientIrmNumber: int = 0
    appName: str = ''
    opCode: int = 0
    appInputStringVal: List[str] = ()


@dataclass
class DistributedAppResponse(TcBaseObj):
    """
    'DistributedAppResponse' structure is response from the distributed application contains 'Servicedata' and
    'DistributedAppReturnStruct' objects.
    
    :var appResponse: List of 'DistributedAppReturnStruct'  structure object.
    
    :var serviceData: The Service Data with failure message if any.
    """
    appResponse: List[DistributedAppReturnStruct] = ()
    serviceData: ServiceData = None


@dataclass
class DistributedAppReturnStruct(TcBaseObj):
    """
    'DistributedAppReturnStruct' structure is return structure of the distributed application contains output values.
    
    :var appOutputStringVal: List of required output values depend on user input.
    :var errorCodes: List of error code.
    :var errorTexts: List of corressponing error texts for error codes.
    """
    appOutputStringVal: List[str] = ()
    errorCodes: List[int] = ()
    errorTexts: List[str] = ()


"""
MapData
"""
MapData = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2011_06.IncrementalChange import AttributeChangeResponse, StructureChangeResponse, PredecessorChangeResponse, AttachmentChangeResponse, ParentAndChildComponentsResponse
from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow, MECfgLine, ItemRevision
from tcsoa.gen.Internal.StructureManagement._2011_06.VariantManagement import ModularOptionsInput, BOMVariantConfigOptionResponse, ModularOptionsForBomResponse
from tcsoa.gen.Internal.StructureManagement._2011_06.Structure import CreateOrSaveAsPSBOMViewRevisionInput, CreateOrSaveAsPSBOMViewRevisionResponse, GetAllAvailableViewTypesInput, GetAvailableViewTypesResponse
from typing import List
from tcsoa.base import TcService


class IncrementalChangeService(TcService):

    @classmethod
    def getStructureChanges(cls, bomlines: List[BOMLine], fRefresh: bool) -> StructureChangeResponse:
        """
        Gets the changes on the occurrence for the given list of BOMLine objects captured in incremental change (IC)
        context.  All changes are returned not just those from the configured window.
        
        Use cases:
        User needs to find the changes done on the occurrence in incremental change (IC) context, for a given set of
        BOMLine objects.  The input to the operation is a list of BOMLine objects.  The output is list of
        IncrementalChangeElement (ICE) objects.
        """
        return cls.execute_soa_method(
            method_name='getStructureChanges',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='IncrementalChange',
            params={'bomlines': bomlines, 'fRefresh': fRefresh},
            response_cls=StructureChangeResponse,
        )

    @classmethod
    def getAttachmentChanges(cls, attachmentlines: List[MECfgLine], fRefresh: bool) -> AttachmentChangeResponse:
        """
        This operation gets changes on datasets, forms and relations under incrementalchange (IC) context.  All changes
        and not just the ones configured in the window are returned.
        
        Use cases:
        The user needs to get attachment data changes on datasets, forms, and/or relations under incremental change
        context. The input to this operation is a list of attachment lines.  The output is a list of attachment change
        information like the type of IncrementalChangeElement, and in which incremental change revision context.
        """
        return cls.execute_soa_method(
            method_name='getAttachmentChanges',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='IncrementalChange',
            params={'attachmentlines': attachmentlines, 'fRefresh': fRefresh},
            response_cls=AttachmentChangeResponse,
        )

    @classmethod
    def getAttributeChanges(cls, attributeOccLines: List[BOMLine], fRefresh: bool) -> AttributeChangeResponse:
        """
        This operation gets the changes on the occurrence attribute (notes, conditions) tracked under incremental
        change context.
        
        Use cases:
        The user needs to get the attribute changes on the occurrence under incremental change (IC) context for one or
        many BOMLine objects.  The user calls the operation 'getAttributeChanges' with a list of input BOMLine objects.
         The output returned is a list of attribute information for each BOMLine object.
        """
        return cls.execute_soa_method(
            method_name='getAttributeChanges',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='IncrementalChange',
            params={'attributeOccLines': attributeOccLines, 'fRefresh': fRefresh},
            response_cls=AttributeChangeResponse,
        )

    @classmethod
    def getParentAndChildComponents(cls, icRevision: ItemRevision) -> ParentAndChildComponentsResponse:
        """
        Gets the parent and child components for the changes on occurrences, occurrence attributes, datasets and/or
        forms under an incremental change context.
        
        Use cases:
        The user wants to get the parent and child object for all the changes made in the context of a given
        incremental change revision.  The input to this operation is the incremental change revision.  The output is a
        list of structures containing parent and child objects for each IncrementalChangeElement object.
        """
        return cls.execute_soa_method(
            method_name='getParentAndChildComponents',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='IncrementalChange',
            params={'icRevision': icRevision},
            response_cls=ParentAndChildComponentsResponse,
        )

    @classmethod
    def getPredecessorChanges(cls, predecessorLines: List[BOMLine], fRefresh: bool) -> PredecessorChangeResponse:
        """
        For a given set of BOMLine objects, 'getPredecessorChanges' returns predecessor changes captured in incremental
        change (IC) context.  All changes and not just the ones configured in the window are returned.
        are returned.
        Note:
        A predecessor represents a sequence relationship between two processes or operations. An occurrence, which
        represents a process or operation, has an attribute predList to store a list of predecessors for it.  When a
        new predecessor is added for a process or operation, it appends an occurrence to the predList. Similarly when a
        predecessor is removed for a process or operation, it removes the occurrence from the predList. To manage
        changes to predecessors of a process or operation, an IncrementalChangeElement (ICE) object will be created
        whenever a predecessor has been added or removed in the context of incremental change (IC).
        
        
        Use cases:
        The user wants to get the predecessor changes tracked by incremental change (IC) context.  A BOMLine could have
        multiple predecessor changes.  A list of BOMLine objects is input to this operation and a list of predecessor
        IncrementalChangeElement (ICE) object(s) are returned.
        """
        return cls.execute_soa_method(
            method_name='getPredecessorChanges',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='IncrementalChange',
            params={'predecessorLines': predecessorLines, 'fRefresh': fRefresh},
            response_cls=PredecessorChangeResponse,
        )


class StructureService(TcService):

    @classmethod
    def createOrSavePSBOMViewRevision(cls, createOrSavePSBOMViewRevisionInputs: List[CreateOrSaveAsPSBOMViewRevisionInput]) -> CreateOrSaveAsPSBOMViewRevisionResponse:
        """
        This operation is to create a new PSBOMViewRevision object or perform a Save As based on existing
        PSBOMViewRevision object.
        
        Use cases:
        The user wants to create a new PSBOMViewRevision object or want to perform Save As based on existing
        PSBOMViewRevision object.
        """
        return cls.execute_soa_method(
            method_name='createOrSavePSBOMViewRevision',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'createOrSavePSBOMViewRevisionInputs': createOrSavePSBOMViewRevisionInputs},
            response_cls=CreateOrSaveAsPSBOMViewRevisionResponse,
        )

    @classmethod
    def getAvailableViewTypes(cls, getAvailableViewTypesInputs: List[GetAllAvailableViewTypesInput]) -> GetAvailableViewTypesResponse:
        """
        This operation is to query the available PSBOMView types objects for the given ItemRevision object. The output
        available list will exclude the PSBOMView types which are already created on given ItemRevision object.
        
        Use cases:
        This operation is called when user wants to know all the available PSBOMView type objects for a given
        ItemRevision object which are not yet used.
        """
        return cls.execute_soa_method(
            method_name='getAvailableViewTypes',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'getAvailableViewTypesInputs': getAvailableViewTypesInputs},
            response_cls=GetAvailableViewTypesResponse,
        )


class VariantManagementService(TcService):

    @classmethod
    def getBOMVariantConfigOptions(cls, bomWindow: BOMWindow, bomLine: BOMLine) -> BOMVariantConfigOptionResponse:
        """
        This operation will provide currently applied variant configuration for a given BOMLine object in its
        respective BOMWindow object. This will output variant configuration information for both modular variants as
        well as legacy variants. Output Variant configuration consists of variant option name, item id, type of variant
        option (classic or modular), currently set value, currently value how-set and option description.
        
        Use cases:
        This operation should be used when user wants to know currently applied variant configuration on a BOMLine in a
        BOMWindow.
        """
        return cls.execute_soa_method(
            method_name='getBOMVariantConfigOptions',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='VariantManagement',
            params={'bomWindow': bomWindow, 'bomLine': bomLine},
            response_cls=BOMVariantConfigOptionResponse,
        )

    @classmethod
    def getModularOptionsForBom(cls, modules: List[ModularOptionsInput]) -> ModularOptionsForBomResponse:
        """
        This operation will provide all modular variant options defined or available on given BOMLine (also called as
        Module). Multiple BOMLine objects can be given as input, and operation will return modular variant options
        defined for each input BOMLine object.
        
        Use cases:
        It should be used when user wants to get all defined or available Modular Variant Option information for one or
        some set of BOMLine object(s).
        """
        return cls.execute_soa_method(
            method_name='getModularOptionsForBom',
            library='Internal-StructureManagement',
            service_date='2011_06',
            service_name='VariantManagement',
            params={'modules': modules},
            response_cls=ModularOptionsForBomResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Vendormanagement._2008_05.VendorManagement import LineItemPropsWithType, ItemIDInput, ItemIDResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class VendorManagementService(TcService):

    @classmethod
    def getItemIDwithContext(cls, input: List[ItemIDInput]) -> ItemIDResponse:
        """
        This service operation concatenates external object id and context id with hyphen to generate a new item id,
        provided length of concatenated id is less than the desired item id length. This operation also returns
        existing Item objects that have the generated IDs.
        The intent of this operation is to support import of Item objects from external systems. This service operation
        concatenates external object id and context id with hyphen to generate a new item id, provided length of
        concatenated id is less than the desired item id length. This operation also returns existing Item objects that
        have the generated IDs.
        The intent of this operation is to support import of Item objects from external systems.
        """
        return cls.execute_soa_method(
            method_name='getItemIDwithContext',
            library='Internal-Vendormanagement',
            service_date='2008_05',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ItemIDResponse,
        )

    @classmethod
    def createOrUpdateLineItems(cls, properties: List[LineItemPropsWithType], bidPackage: BusinessObject) -> ServiceData:
        """
        This service operation creates or updates a group of BidPackageLineItem objects in the context of the mentioned
        BidPackage. This operation allows the user to find or create a set of BidPackageLineItem objects based on the
        input data. It first tries to find the existence of the specified BidPackageLineItem for the specified
        BidPackage. If the specified BidPackageLineItem is found, then the BidPackageLineItem data will be updated.
        Otherwise, it will create the specified BidPackageLineItem and associate it to the specified
        BidPackageRevision. If the specified BidPackageLineItem objects exist and any of the input attribute values
        differ from those already set, the BidPackageLineItem objects will be updated.
        The operation is now deprecated. It is now replaced by
        'Teamcenter::Soa::Vendormanagement::VendorManagement::createOrUpdateLineItemsWithType.'
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateLineItems',
            library='Internal-Vendormanagement',
            service_date='2008_05',
            service_name='VendorManagement',
            params={'properties': properties, 'bidPackage': bidPackage},
            response_cls=ServiceData,
        )



from tcsoa.gen.Internal_OccMgmt._2020_12.services import EffectivityManagementService as imp0
from tcsoa.gen.Internal_OccMgmt._2020_05.services import ImportExportService as imp1
from tcsoa.base import TcService


class EffectivityManagementService(TcService):
    getEffectivity = imp0.getEffectivity
    setEffectivity = imp0.setEffectivity


class ImportExportService(TcService):
    getMappingGroupInfo = imp1.getMappingGroupInfo
    importExcelAndUpdateMappingGrp = imp1.importExcelAndUpdateMappingGrp
    importExcelAndUpdateMappingGrpAsync = imp1.importExcelAndUpdateMappingGrpAsync



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ClosureRule, RuntimeBusinessObject, AssemblyArrangement, WorkspaceObject, PSViewType, RevisionRule, Awb0Element, Awb0ProductContextInfo, ItemRevision
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import SearchSortCriteria2, Criteria, SearchFilter2
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12.OccurrenceManagement import EffectivityRange, SourceContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12.OccurrenceManagement import OccurrenceCursor, OccurrenceInfo6, UserContextInfo, FocusOccurrenceInput
from typing import Dict, List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05.OccurrenceManagement import OccurrenceSortInput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class OccurrenceFilterInput2(TcBaseObj):
    """
    'OccurrenceFilterInput' specifies the information about recipe to search and filter the result.
    
    :var recipe: Recipe to provide search criteria.
    :var searchFilterCategories: A list of search filter categories.
    :var searchFilterMap: A map (string,list of SearchFilter) containing the list of search filters for each search
    filter field.
    :var searchSortCriteria: A list of criterion to use to sort the filtered results.
    :var searchFilterFieldSortType: Type of sorting that needs to be performed on the search results. The fields can be
    sorted alphabetically in ascending order or in priority order from highest to lowest. The valid values are
    "Alphabetical" &amp; "Priority".
    :var fetchUpdatedFilters: A boolean flag to control filter data population in response. If true, response contains
    updated filters. If false, response does not contain any filter data. E.g. in case of scrolling down the contents,
    this flag can be set as false not to return updated filter as there won't be any updates.
    """
    recipe: List[Criteria] = ()
    searchFilterCategories: List[SearchFilterField3] = ()
    searchFilterMap: SearchFilterMap3 = None
    searchSortCriteria: List[SearchSortCriteria2] = ()
    searchFilterFieldSortType: str = ''
    fetchUpdatedFilters: bool = False


@dataclass
class OccurrenceFilterOutput2(TcBaseObj):
    """
    'OccurrenceFilterOutput2' specifies the output search criteria and updated filters.
    
    :var recipe: Updated recipe including the user specified filters that were provided in 'OccurrenceFilterInput'.
    :var searchFilterCategories: A list of search filter categories.
    :var searchFilterMap: A map ( String, list of 'SearchFilter2' ) of search filter categories ( String ) with their
    corrosponding list of search filters ( 'SearchFilter2' ). Valid keys are' Ptn0PartitionScheme' and 'Design
    Component'.
    :var defaultCategoryDisplayCount: Number of categories display count.
    """
    recipe: List[Criteria] = ()
    searchFilterCategories: List[SearchFilterField3] = ()
    searchFilterMap: SearchFilterMap3 = None
    defaultCategoryDisplayCount: int = 0


@dataclass
class OccurrenceInfo3(TcBaseObj):
    """
    OccurrenceInfo3 is the information about occurrence that is part of the configuration result and passing the filter
    criteria.
    
    :var occurrence: Instance of the BusinessObject that represents a line in the structure.
    :var occurrenceId: Session Recoverable UID of the configured occurrence.
    :var stableId: Stable UID of occurrence. This strictly used for maintaining expansion state of nodes in tree view.
    :var displayName: Display name of the configured occurrence.
    :var numberOfChildren: Number of children of the configured occurrence.
    :var underlyingObjectType: Internal name of the underlying object that occurrence presents.
    """
    occurrence: BusinessObject = None
    occurrenceId: str = ''
    stableId: str = ''
    displayName: str = ''
    numberOfChildren: int = 0
    underlyingObjectType: str = ''


@dataclass
class OccurrencesData3(TcBaseObj):
    """
    The 'OccurrencesData3' supplies the product and the configuration rules to configure the product and get a page of
    occurrences along with effective column configuration. Optionally, filters and sorting criteria may be provided to
    filter or sort the resulting occurrences.
    
    :var product: The top-level product. It may be an ItemRevision representing a product or a Mdl0odelElement such as
    a Cpd0CollaborativeDesign.
    :var parentElement: Parent occurrence for which child occurrences is to be retrieved.
    :var focusOccurrenceInput: Contains input information required to set the focusChildOccurrence.
    :var config: Specifies the complete set of configuration rules including RevisionRule, Effectivity, ClosureRule and
    VariantRule to configure content.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var filter: A recipe to search and filter occurrences and sort the results.
    :var sortCriteria: Criteria to sort the occurrences based on intetnal property name and sorting order e.g. "ASC"
    and "DESC".
    :var requestPref: A map (string, string) of preference names and value pairs. Allowed preference names are:
    "includeInterfaces" , and "includeUnconfigured". Allowed values are: True/False. Keys and values are case
    sensitive. When preference "includeInterface" with value true is specified the operation shall return instances of
    Awb0Interface objects. When preference "includeUnconfigured" with value true is specified the operation shall
    return instances of unconfigured Awb0Element objects.
    :var expansionCriteria: Criteria to expand node
    """
    product: BusinessObject = None
    parentElement: str = ''
    focusOccurrenceInput: FocusOccurrenceInput = None
    config: OccurrenceConfigInput = None
    cursor: OccurrenceCursor = None
    filter: OccurrenceFilterInput2 = None
    sortCriteria: OccurrenceSortInput = None
    requestPref: RequestPreference10 = None
    expansionCriteria: ExpansionCriteria = None


@dataclass
class OccurrencesResp3(TcBaseObj):
    """
    Contains the resulting occurrences found for product from which retrieval was requested. This data structure also
    returns a cursor to be sent back to the subsequent call to getOccurrences3 in order to rerieve more pages of
    results.
    
    :var rootProductContext: The Awb0ProductContextInfo for the root occurrence.
    :var parentProductContext: The Awb0ProductContextInfo for the parent occurrence.
    :var parentChildrenInfos:  List of parent and its children information.
    :var requestPref:  A map (string, string) of preference names and value pairs. Allowed preference names are:
    "includeInterfaces" , and "includeUnconfigured". Allowed values are: True/False. Keys and values are case
    sensitive. When preference "includeInterface" with value true is specified the operation shall return instances of
    Awb0Interface objects. When preference "includeUnconfigured" with value true is specified the operation shall
    return instances of unconfigured Awb0Element objects.
    :var serviceData: Contains the list of all BusinessObjects that make up the output, as well as any errors that
    might have occurred as part of the service invocation.
    :var parentOccurrence: The parent occurrence for list of childOccurrences.
    :var focusProductContext:  The Awb0ProductContextInfo for the focus occurrence.
    :var focusChildOccurrence: The focus child occurrence for which sibling occurrences are desired.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    :var userWorkingContextInfo: User's working context information for opened content.
    :var filter: Updated search filter information.
    :var elementToPCIMap:  Map containing pair of Root Occurrence (Awb0Element) to its corresponding
    Awb0ProductContextInfo object in Product Set container.
    :var sourceContextToInfoMap: Map(WorkspaceObject, SourceContextInfo) containing pair of Source Context Object to
    its corresponding configuration information.
    """
    rootProductContext: Awb0ProductContextInfo = None
    parentProductContext: Awb0ProductContextInfo = None
    parentChildrenInfos: List[ParentChildrenInfo3] = ()
    requestPref: RequestPreference10 = None
    serviceData: ServiceData = None
    parentOccurrence: OccurrenceInfo6 = None
    focusProductContext: Awb0ProductContextInfo = None
    focusChildOccurrence: OccurrenceInfo6 = None
    cursor: OccurrenceCursor = None
    userWorkingContextInfo: UserContextInfo = None
    filter: OccurrenceFilterOutput2 = None
    elementToPCIMap: ElementToProductContextInfoMap4 = None
    sourceContextToInfoMap: SourceContextToInfoMap3 = None


@dataclass
class ParentChildrenInfo3(TcBaseObj):
    """
    List of parent and its children information.
    
    :var parentInfo: Information of parent occurrence.
    :var childrenInfo: Information of children occurrences corresponding to parentInfo.
    :var cursor: Cursor that represents point to which the results have been fetched. This field is used for subsequent
    calls.
    """
    parentInfo: OccurrenceInfo3 = None
    childrenInfo: List[OccurrenceInfo3] = ()
    cursor: OccurrenceCursor = None


@dataclass
class RemoveInContextOverrides(TcBaseObj):
    """
    Contains the Awb0Element object and the overridden property which needs to be removed.
    
    :var element: The Awb0Element whose overridden property value to be removed.
    :var propertyName: Internal name of the property of Awb0Element for which override to be removed.
    :var requestPref: Map of preference names and values pairs(string / list of string ). Preference names and values
    are case sensitive.
    """
    element: RuntimeBusinessObject = None
    propertyName: str = ''
    requestPref: RequestPreference10 = None


@dataclass
class SearchFilterField3(TcBaseObj):
    """
    'SearchFilterField3' contains the resulting configured products found, that contain the queried object. This data
    structure also returns a cursor which can be sent back to a call to in order to rerieve more pages of results.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    :var editable: If true, UI renders an editable text box instead of filter values. E.g. when 'categoryType' is
    "Attribute", in UI instead of displaying filter value a text field should be rendered.
    :var quickSearchable: If true, UI rendes a quick search text box. E.g. when 'categoryType' is "Partition", in UI a
    quick search text box is rendered along with all filter values.
    :var categoryType: The type of the filter category. Valid values are "StringMatch", "Partition", "Attribute"
    ,"ClosureRule", "SelectedElement"," BoxZone", "Proximity"," PlaneZone"
    :var isHierarchical: A boolean to indicate if hiearchical display is required for the filter values that belong to
    this categroy. If true, the filter values must have a hiearchical display in the client. Otherwise the filter
    values are shown as a flat list. Default value of this parameter is false.
    :var isMultiSelect:  If true, multiple filters can be selected within this category. Otherwise only single filter
    can be selected. Default value of this parameter is true.
    :var endIndex: The ending index value of the list. This field is applicable only if the 'searchFilterType' field is
    set to "StringFilter" and has list of values.
    :var endReached: If true, the last page of the results has been reached.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0
    editable: bool = False
    quickSearchable: bool = False
    categoryType: str = ''
    isHierarchical: bool = False
    isMultiSelect: bool = False
    endIndex: int = 0
    endReached: bool = False


@dataclass
class SubsetResponse2(TcBaseObj):
    """
    'SubsetResponse2' contains a output subset recipe and filters for every given product context input.
    
    :var filterOut: A list of filter comprising the recipes and filters for the given Awb0ProducContextInfo.
    :var serviceData: Contains the list of all BusinessObject(s) that make up the output, as well as any errors that
    might have ocurred as part of the service invocation.
    """
    filterOut: List[OccurrenceFilterOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class DetachObjectsInputData(TcBaseObj):
    """
    Represents all required parameters to detach secondary objects with primary object.
    
    :var primaryObj: The primary Awb0Element object for the relation to detach.
    :var secondaryObjs: A list of secondary WorkspaceObject objects for the relations to detach.
    """
    primaryObj: Awb0Element = None
    secondaryObjs: List[WorkspaceObject] = ()


@dataclass
class ExpansionCriteria(TcBaseObj):
    """
    Crietria for node expansion.
    
    :var expandBelow: If true node will be expanded. If false node will not be expanded.
    :var loadTreeHierarchyThreshold: Maximum number of nodes to fetch on any node.
    :var scopeForExpandBelow: Uid of the node.
    :var levelNExpand: Number of levels to be expanded for any node.
    """
    expandBelow: bool = False
    loadTreeHierarchyThreshold: int = 0
    scopeForExpandBelow: str = ''
    levelNExpand: int = 0


@dataclass
class FilterSearchCriteria2(TcBaseObj):
    """
    'FilterSearchCriteria2' provides information related to the value entered in search filter field, as well as the
    category related details.
    
    :var categoryType: The category type under which search is requested. Valid values are:" Partition", "Attribute" ,
    "ClosureRule", "SelectedElement", "BoxZone", "Proximity", "PlaneZone"," StringMatch".
    :var categoryInternalName: The name of the category.
    :var searchString: User entered search text for finding the filters.
    :var endIndex: Next index value of list.
    :var pageSize: Indicates the maximum number of values that can be returned in one service call.
    """
    categoryType: str = ''
    categoryInternalName: str = ''
    searchString: str = ''
    endIndex: int = 0
    pageSize: int = 0


@dataclass
class FindMatchingFilterInput2(TcBaseObj):
    """
    Input for 'findMatchingFilters'.
    
    :var productInfo: Product context info.
    :var appliedCriteria: List of criteria which are already applied. This list is used to set selected parameter in
    returned 'SearchFilter2'.
    :var searchCriteria: search criteria
    """
    productInfo: Awb0ProductContextInfo = None
    appliedCriteria: List[Criteria] = ()
    searchCriteria: FilterSearchCriteria2 = None


@dataclass
class FindMatchingFilterOutput2(TcBaseObj):
    """
    'FindMatchingFilterOutput2' provides output information related to the value entered in search filter field, as
    well as the category related details.
    
    :var searchFilterMap: Contains the map (Filter category namestring, list of 'SearchFilter2' ) of category name to
    its display values.
    :var endIndex: The ending index value of the list. This field is applicable only if the 'searchFilterType' field is
    set to "StringFilter" and has list of values.
    :var endReached: If true, the last page of the results has been reached.
    """
    searchFilterMap: SearchFilterMap3 = None
    endIndex: int = 0
    endReached: bool = False


@dataclass
class FindMatchingFilterResponse2(TcBaseObj):
    """
    'FindMatchingFilterResponse' contains output categories and their display values organized based on the search sort
    criteria specified in the input.
    
    :var filterOut: A list of filter output comprising the list of categories and assocaited filter values for the
    given Awb0ProducContextInfo.
    :var serviceData: Contains the list of all BusinessObjects that make up the output, as well as any errors that
    might have ocurred as part of the service invocation.
    """
    filterOut: List[FindMatchingFilterOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class InfoForAddElemData3(TcBaseObj):
    """
    This structure contains information to identify type that can be added as child of input parentElement or
    productContext.
    
    :var parentElement: Awb0Element under which another Awb0Element to be added as child. This is specified if
    Awb0Element needs to be added under parent context.
    :var fetchAllowedOccRevTypes: If true, operation returns allowed occurrence revision (Fnd0AbstractOccRevision)
    types; otherwise, it returns allowed child Item types. The allowed occurrence revision types can be specified using
    preference TCAllowedOccRevTypes_<ParentItemType>.
    """
    parentElement: Awb0Element = None
    fetchAllowedOccRevTypes: bool = False


@dataclass
class OccurrenceConfigInput(TcBaseObj):
    """
    OccurrenceConfigInput specifies the complete set of configuration rules including RevisionRule objects ,
    Effectivity row data (this may or may not contain effectivity intents), ClosureRule and VariantRule to configure
    content.
    
    :var productContext: Awb0ProductContextInfo containing configuration information.
    :var revisionRule: RevisionRule to use in order to configure the product.
    :var occurrenceScheme: Occurrence scheme associated with product e.g CollaborativeDesign.
    :var sourceContext: Source Context (Fnd0RecipeContainer or Mdl0BaselineRevision) associated with product e.g.
    CollaborativeDesign.
    :var changeContext: Runtime Business object fnd0ChangeContext that holds the Change Context to be set and
    corresponding Change Configuration Mode.
    :var appliedArrangement: The arrangement applied on the top level product.
    :var closureRule: ClosureRule used to configure the product.
    :var viewType: Type of View Type applied to the product.
    :var serializedRevRule: Transient RevisionRule to use in order to configure the product.
    :var effectivityDate: Specifies the date effectivity to use in order to configure the product.
    :var effectivityGroups: List of Fnd0EffGrpRevision objects to configure the product.
    :var unitNo: Unit number to use to configure the product.
    :var endItem: Specifies the end Item which governs the  unit effectivity.
    :var variantRules: List of VariantRule(s) or StoredOptionSet(s) to use in order to configure the product.
    :var svrOwningProduct: This indiciates the product(ItemRevision or CollaborativeDesign) associated with saved
    VariantRule. This information is not used when there are more than one VariantRule supplied.
    :var effectivityRanges: A list of effectivity data with effectivity intents.
    """
    productContext: Awb0ProductContextInfo = None
    revisionRule: RevisionRule = None
    occurrenceScheme: BusinessObject = None
    sourceContext: WorkspaceObject = None
    changeContext: RuntimeBusinessObject = None
    appliedArrangement: AssemblyArrangement = None
    closureRule: ClosureRule = None
    viewType: PSViewType = None
    serializedRevRule: str = ''
    effectivityDate: datetime = None
    effectivityGroups: List[ItemRevision] = ()
    unitNo: int = 0
    endItem: BusinessObject = None
    variantRules: List[WorkspaceObject] = ()
    svrOwningProduct: BusinessObject = None
    effectivityRanges: List[EffectivityRange] = ()


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive. The allowed preference names are includeInterfaces. The allowed values are True/False.
"""
RequestPreference10 = Dict[str, List[str]]


"""
'SearchFilterMap3' is a map (Filter category name, string list of 'SearchFilter2') containing the list of search filters for each search filter field based on the search results.
"""
SearchFilterMap3 = Dict[str, List[SearchFilter2]]


"""
Map(WorkspaceObject, SourceContextInfo) containing pair of Source Context Object to its corresponding configuration information
"""
SourceContextToInfoMap3 = Dict[WorkspaceObject, SourceContextInfo]


"""
Map containing Awb0Element to its respective Awb0ProductContextInfo object.
"""
ElementToProductContextInfoMap4 = Dict[Awb0Element, Awb0ProductContextInfo]



from __future__ import annotations

from tcsoa.gen.Internal.Visualization._2018_11.StructureManagement import ProductStructureIdInput, GetProductStructureIdResponse
from typing import List
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def getStructureIdFromRecipe(cls, productStructureIdInput: List[ProductStructureIdInput]) -> GetProductStructureIdResponse:
        """
        This operation retrieves the ID that is used to identify aproduct structure from Teamcenter that has been
        indexed and stored in the visualization data server (VDS) for accelerated visualization performance. The ID is
        a unique identifier which gets written into the MMP file (the file used to transport the indexed product
        structure to the VDS) as the product ID when the product is indexed. The Visualization Data server (VDS)
        maintains indexed product structure information for different products and configurations identified by these
        product IDs.
        
        The Visualization client invokes this operation to check if the product and configuration information for  the
        current visualization request matches that for any of the product structures that have been indexed in the VDS.
         If the product and configuration has been indexed in VDS, then this operation returns the product ID to the
        visualization client.  The visualization client uses this information to initiate a session with the VDS which
        allows the visualization client to retrieve the structure information from VDS instead of from Teamcenter.
        
        For example, the VDS may have indexed product structure data with structure ID "UidOfBomIndexAdminData1" for
        product1 with the RevisionRule  "Latest Working" and VariantRule "vrule1", "vrule2", "vrule3".  The VDS may
        also have indexed product structure data with ID "UidOfBomIndexAdminData2" for  product2 with RevisionRule 
        "Latest Released" and VariantRule "vrule1", "vrule2", "vrule3". If the Visualization client wants to display
        product1 with "Latest working" RevisionRule and VariantRule  "vrule2", it invokes this operation, gets the
        product id "UidOfBomIndexAdminData1" and uses this product id to get the structure information from the VDS.
        
        Use cases:
        VDS is deployed on a LAN close to the visualization client. The VDS acts as a product structure server for the
        visualization clients. The VDS is configured to periodically communicate to the Tcserver and retrieve the
        indexed product structure information.
        - VDS boots and invokes getIndexedProducts  and gets the products that are to be indexed.
        - VDS invokes getStructureFiles operation on each of the indexed products to get the product structure
        information represented in a binary file (*.mmp).
        - If the product structure information is returned as a complete file and a set of delta files then VDS merges
        the delta file into the complete structure to get the current complete product structure .
        - If only delta product structure file is returned then those files are merged with the complete product
        structure data the VDS already has.
        - The VDS is now ready to serve product structure to visualization clients.
        - The user selects an object in Teamcenter to be viewed in the visualization client. A recipe object is
        received by the visualization client from Teamcenter via the VVI generation process. This is the process to
        communicate to the viewer what the user has selected.
        - The visualization client invokes the getStructureIdFromRecipe to determine which product structure data in
        VDS contains the requested product structure information by passing the recipe object that encapsulates the
        product to configure and the configuration information. There are numerous of object types supported by the
        visualization client. Currently this service API supports the Awb0ProductContextInfo or VisStructureContext
        types.
        - The Visulalization client initiates a session with  the VDS using the Product ID returned by
        getStructureIdFromRecipe.
        - The VDS servers a spatial hierarchy file and corresponding product structure data for this structure ID  to
        the visualization client to enable  massive model visualization (MMV).
        
        """
        return cls.execute_soa_method(
            method_name='getStructureIdFromRecipe',
            library='Internal-Visualization',
            service_date='2018_11',
            service_name='StructureManagement',
            params={'productStructureIdInput': productStructureIdInput},
            response_cls=GetProductStructureIdResponse,
        )



from tcsoa.gen.AWS2._2018_12.services import RequirementsManagementService as imp0
from tcsoa.gen.AWS2._2016_12.services import UiConfigService as imp1
from tcsoa.gen.AWS2._2017_06.services import UiConfigService as imp2
from tcsoa.base import TcService


class RequirementsManagementService(TcService):
    createTracelinks = imp0.createTracelinks


class UiConfigService(TcService):
    getUIConfigs2 = imp1.getUIConfigs2
    getUIConfigs3 = imp2.getUIConfigs3



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, Participant
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ParticipantInfo(TcBaseObj):
    """
    ParticipantInfo structure contains information about WorkspaceObject and list of Participant objects.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var wso: WorkspaceObject whose participants are modified.
    :var participants: A list of Participant objects.
    :var additionalData: A map (string/list of strings) to send additional information. This is provided for future use
    and will be implemented in future. For example, user can provide comments that can be sent as "COMMENTS" as key and
    user entered comments as values.
    """
    clientId: str = ''
    wso: WorkspaceObject = None
    participants: List[Participant] = ()
    additionalData: KeyValueMap = None


@dataclass
class ParticipantInfoInput(TcBaseObj):
    """
    ParticipantInfoInput structure contains information about WorkspaceObject and list of ParticipantInput structures.
    
    :var wso: The WorkspaceObject on which the particpant is to be added.
    :var additionalData: A map (string/list of strings) to send additional information. This is provided for future use
    and will be implemented in future. For example, user can provide comments that can be sent as "COMMENTS" as key and
    user entered comments as values.
    :var participantInputData: A list of ParticipantInput structures. Each element will contain the assignee and
    corresponding participant type.
    """
    wso: WorkspaceObject = None
    additionalData: KeyValueMap = None
    participantInputData: List[ParticipantInput] = ()


@dataclass
class ParticipantInfoOutput(TcBaseObj):
    """
    ParticipantInfoOutput structure contains information about list of WorkspaceObject objects on which the participant
    failed to get reassigned.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var wsoList: A list of WorkspaceObject objects on which the participants failed to get reassigned.
    :var additionalData: A map (string/list of strings) to send additional information. This is provided for future use
    and will be implemented in future. For example, user can provide comments that can be sent as "COMMENTS" as key and
    user entered comments as values.
    """
    clientId: str = ''
    wsoList: List[WorkspaceObject] = ()
    additionalData: KeyValueMap = None


@dataclass
class ParticipantInput(TcBaseObj):
    """
    ParticipantInput structure contains information about assignee and participant type.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var assignee: The participant which can be a User, GroupMember or a ResourcePool.
    :var participantType: Corresponding participant type to which the assignee will be added. For example
    PROPOSED_RESPONSIBLE_PARTY, PROPOSED_REVIEWERS, CHANGE_SPECIALIST1, CHANGE_SPECIALIST2, CHANGE_SPECIALIST3,
    CHANGE_REVIEW_BOARD, CHANGE_IMPLEMENTATION_BOARD, ANALYST, REQUESTOR.
    """
    clientId: str = ''
    assignee: BusinessObject = None
    participantType: str = ''


@dataclass
class ParticipantResponse(TcBaseObj):
    """
    ParticipantResponse structure contains information about list of ParticipantInfo structures and service data.
    
    :var output: A list of ParticipantInfo structure.
    :var serviceData: The service data.
    """
    output: List[ParticipantInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ReassignParticipantInfoInput(TcBaseObj):
    """
    ReassignParticipantInfoInput structure contains information about WorkspaceObject, old value of assignee, new value
    of assignee, participant types and comments.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var wsoList: A list of WorkspaceObject to reassign participant roles.
    :var fromAssignee: The business object that can be User, GroupMember or ResourcePool to be replaced with the
    toAssignee business object.
    :var toAssignee: The business object that can be User, GroupMember or ResourcePool to replace the fromAssignee
    business object.
    :var participantTypes: A list of Participant types to be reassigned.
    :var allParticipantTypes: If true, all the participant roles for the fromAssignee will be considered for
    reassignment. The values provided in the participantTypes list will be ignored. If false, only the participant
    roles provided in the participantTypes list will be considered.
    :var comment: A comment provided during reassignment.
    :var additionalData: A map (string/list of strings) to send additional information. This is provided for future use
    and will be implemented in future. For example, user can provide comments that can be sent as "COMMENTS" as key and
    user entered comments as values.
    """
    clientId: str = ''
    wsoList: List[WorkspaceObject] = ()
    fromAssignee: BusinessObject = None
    toAssignee: BusinessObject = None
    participantTypes: List[str] = ()
    allParticipantTypes: bool = False
    comment: str = ''
    additionalData: KeyValueMap = None


@dataclass
class ReassignParticipantResponse(TcBaseObj):
    """
    ReassignParticipantResponse structure contains information about list of ParticipantInfoOutput structures and
    service data.
    
    :var output: A list of ParticipantInfoOutput structure.
    :var serviceData: The service data.
    """
    output: List[ParticipantInfoOutput] = ()
    serviceData: ServiceData = None


"""
Structure for passing data in form of key and values.
"""
KeyValueMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Internal.Core._2007_06.ProjectLevelSecurity import Filter
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ProjectSmartFolderHierarchyOutput2(TcBaseObj):
    """
    This structure holds smart folder hierarchy information and a project ID of one TC_Project object.
    
    :var projectID: The unique identifier of the TC_Project object.
    :var projectSmartFolderHierarchy: A list  of SmartFolderOutputInfo2 objects.
    """
    projectID: str = ''
    projectSmartFolderHierarchy: List[SmartFolderOutputInfo2] = ()


@dataclass
class ProjectSmartFolderHierarchyOutputResponse2(TcBaseObj):
    """
    This structure contains a list of ProjectSmartFolderHierarchyOutput2 structures and a standard ServiceData
    element.If the project id does not exist in the system then error code 101007: the project ID is invalid is
    returned.
    
    :var output: List of ProjectSmartFolderHierarchyOutput2 structures.
    :var serviceData: A  standard ServiceData.
    """
    output: List[ProjectSmartFolderHierarchyOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class SmartFolderOutputInfo2(TcBaseObj):
    """
    This structure holds smart folder hierarchy information.
    
    :var parentName: The name of the parent smart folder node of the current hierarchy.
    :var parentInternalName: The internal name of the parent smart folder node of the current hierarchy.
    :var name: The name of the current smart folder node.
    :var internalName: The internal name of the current smart folder node.
    :var filters: A list of filiters for this smart folder hierarchy.
    :var isLeaf: The flag indicating if this is the last child of the hierarchy.
    """
    parentName: str = ''
    parentInternalName: str = ''
    name: str = ''
    internalName: str = ''
    filters: List[Filter] = ()
    isLeaf: bool = False



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12.OccurrenceManagement import AddObjectData, OccurrencesData, AddObjectResp, OccurrencesResp
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12.Markup import ApplyMarkupData
from tcsoa.gen.BusinessObjects import BusinessObject, Awb0Element
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12.Compare import CompareOptionsResponse, CompareContentData2, CompareResp2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def addObject(cls, input: AddObjectData) -> AddObjectResp:
        """
        This operation adds an object to a product as specified in Awb0ProductContextInfo or parent Awb0Element and
        creates Awb0Element representing newly created object. The new object is added as a sibling to the selected
        Awb0Element if that is provided in input. The Awb0ProductContextInfo and parent Awb0Element can be retrieved
        using getOccurrences7.
        
        Use cases:
        User loads the product structure in Content. The Content  is used for smart navigation of occurrences in a
        structure. User selects an occurrence in the Content and clicks on Add button. This actions will add a new
        occurrence as a child or sibling of the selected occurrence.
        """
        return cls.execute_soa_method(
            method_name='addObject',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_12',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=AddObjectResp,
        )

    @classmethod
    def getOccurrences(cls, inputData: OccurrencesData) -> OccurrencesResp:
        """
        Retrieves the page of configured occurrences for given the top-level product and configuration parameters as
        input. The service provides the facility to optionally filter and sort the result by additional filters and
        sorting criteria that may be provided as input. The output also contains a cursor that defines the place to
        start the next page location. The cursor must be passed back in to any subsequent call to get the next page of
        occurrences.
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException
        """
        return cls.execute_soa_method(
            method_name='getOccurrences',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_12',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=OccurrencesResp,
        )


class MarkupService(TcService):

    @classmethod
    def cancelMarkup(cls, element: Awb0Element, markups: List[BusinessObject]) -> ServiceData:
        """
        This operation cancels the proposed markup changes for input element. This operation returns updated or removed
        Awb0Element objects. If the Awb0Element objects were created  as a markup proposal then the Awb0Element objects
        are removed.  If Awb0Element objects had property change markup proposals then the Awb0Element objects are
        returned as updated objects in ServiceData.
        """
        return cls.execute_soa_method(
            method_name='cancelMarkup',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_12',
            service_name='Markup',
            params={'element': element, 'markups': markups},
            response_cls=ServiceData,
        )

    @classmethod
    def applyMarkup(cls, input: List[ApplyMarkupData]) -> ServiceData:
        """
        This operation applies proposed changes that were stored in the active markup for the specified  Awb0Element
        object, if the evaluate flag is false. If the evaluate flag is true, the write access to perform the save is
        checked and no modification is actually attempted. If the recursive flag is set, then all active Markups in the
        elements under the specified element are evaluated and/or saved.
        """
        return cls.execute_soa_method(
            method_name='applyMarkup',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_12',
            service_name='Markup',
            params={'input': input},
            response_cls=ServiceData,
        )


class CompareService(TcService):

    @classmethod
    def compareContent2(cls, inputData: CompareContentData2) -> CompareResp2:
        """
        This operation compares two contents (can also be referred to as source and target) and returns a output of
        comparison results. The content can be Item BVR structures or a 4th Generation Design or BOM data contained
        within a Subset.
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        
        This operation may raise a ServiceException containing following errors:
        
        126001  :  An internal error has occurred in the Occurrence Management module. 
        126002  :  No Adapter could be found to handle the Request.
        """
        return cls.execute_soa_method(
            method_name='compareContent2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_12',
            service_name='Compare',
            params={'inputData': inputData},
            response_cls=CompareResp2,
        )

    @classmethod
    def getCompareOptions(cls, sourceObject: BusinessObject, targetObject: BusinessObject) -> CompareOptionsResponse:
        """
        This operation retreives the list of options to compare two structures. Structures can be Item BVR structures
        or a 4th Generation Design or BOM data contained within a Subset.
        
        Exceptions:
        >ServiceException:
        This operation may raise a ServiceException containing following errors:
        
        126001 : An internal error has occurred in the Occurrence Management module. 
        126002 : No Adapter could be found to handle the Request.
        126254 : The input "Object" is invalid for compare.
        126255 : The input "Source Object" and "Target Object" combination is invalid for compare.
        """
        return cls.execute_soa_method(
            method_name='getCompareOptions',
            library='Internal-ActiveWorkspaceBom',
            service_date='2018_12',
            service_name='Compare',
            params={'sourceObject': sourceObject, 'targetObject': targetObject},
            response_cls=CompareOptionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanSubscription, ImanEventType, Condition, User, ImanActionHandler
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FindSubscriptionsCriteriaInput(TcBaseObj):
    """
    Contains all the input criteria required to search for Subscriptions.
    
    :var target: The object to monitor for the specified event type.  This can also be Business Object type for class
    based Subscriptions.
    :var subscriber: The User that is subscribing to the event.
    :var eventType: Type of event that will trigger the subscription.
    :var handlers: An ordered list of handlers to execute when processing the Subscription.
    :var condition: The Condition to evaluate when processing this Subscription.  Only valid for class based
    Subscriptions.  Can be NULL.
    """
    target: BusinessObject = None
    subscriber: User = None
    eventType: ImanEventType = None
    handlers: List[ImanActionHandler] = ()
    condition: Condition = None


@dataclass
class GetSubscriptionsCriteriaInput(TcBaseObj):
    """
    Contains all the input criteria required to search for Subscriptions.
    
    :var target: The object to monitor for the specified event type.  This can also be Business Object type for class
    based ImanSubscription.  May be NULL.
    :var subscriber: The User that is subscribing to the event.  May be NULL.
    :var eventType: Type of event that will trigger the subscription.  May be NULL.
    :var handlers: The handlers to execute when processing the Subscription.  May be an empty list.
    :var execTimeBefore: Upper limit of the Subscription execution time.
    :var execTimeAfter: Lower limit of the Subscription execution time.
    :var execImmediately: If true, do not search for Subscriptions with an execution time.
    :var expDateBefore: Upper limit of the Subscription expiration date.
    :var expDateAfter: Lower limit of the Subscription expiration date.
    :var noExpDate: If true, do not search for Subscriptions with an expiration date.
    """
    target: BusinessObject = None
    subscriber: User = None
    eventType: ImanEventType = None
    handlers: List[ImanActionHandler] = ()
    execTimeBefore: datetime = None
    execTimeAfter: datetime = None
    execImmediately: bool = False
    expDateBefore: datetime = None
    expDateAfter: datetime = None
    noExpDate: bool = False


@dataclass
class AttributeComparison(TcBaseObj):
    """
    The overall attribute criteria for a Subscription can be made up of multiple individual attribute comparisons. 
    Each attribute comparison specifies the attribute on the target object to check, the value to compare it against
    and how to do the comparison.
    
    :var mathOperator: The math operator to use when comparing the attribute to the specified value. The MathOperators
    enum comprises of these values: EqualTo, NotEqualTo, GreaterThan, LessThan, GreaterThanEqualTo, LessThanEqualTo.
    :var attributeName: The name of the attribute on the target object to compare against.
    :var attributeValue: The string value to compare against.
    """
    mathOperator: MathOperators = None
    attributeName: str = ''
    attributeValue: str = ''


@dataclass
class AttributeCriteria(TcBaseObj):
    """
    If attribute criteria are specified on a Subscription, these criteria will be evaluated when the Subscription is
    being processed.  This processing involves comparing values in the criteria to the attribute values on the target
    object using a math operator, such as >, < or =.  Multiple attribute comparisons are then grouped using logic
    operators such as AND and OR.  Each AttributeCriteria represents a single attribute comparison and the logical
    operator to use when comparing it to the previous AttributeCriteria entry.
    
    :var logicOperator: The logic operator to use when comparing each AttributeCriteria to the next.  Processing will
    be done in order.  The first value in this list will be ignored.
    :var attributeComparison: A single attribute comparison.
    """
    logicOperator: LogicOperators = None
    attributeComparison: AttributeComparison = None


@dataclass
class ModifySubscriptionInput(TcBaseObj):
    """
    Contains the Subscription object to modify and all of the input values to set on the Subscription.  Supports both
    object and class based Subscriptions.
    
    :var subscription: The ImanSubscription object to modify.
    :var newSubscriptionValues: The new values to set on the Subscription.
    """
    subscription: ImanSubscription = None
    newSubscriptionValues: SubscriptionInput = None


@dataclass
class NotificationDefinition(TcBaseObj):
    """
    For e-mail notification handlers, a NotificationDefinition must be specified.  This structure describes the content
    of the e-mail to be sent as part of the notification Subscription.  This includes the subject, message and object
    properties to display in the message.  The recipients of the message are included in the HandlerParameters
    specified in the Subscription definition.
    
    :var subject: The subject of the notification e-mail.
    :var message: The message body of the notification e-mail.
    :var propertyNames: List of properties to include in the notification e-mail.  The values of these properties will
    be taken from the target object at the time the handler is executed.  These properties are not required.
    """
    subject: str = ''
    message: str = ''
    propertyNames: List[str] = ()


@dataclass
class SubscriptionInput(TcBaseObj):
    """
    Contains all the input criteria required to create a single object or class based Subscription.  The main inputs
    include the target object, subscriber, event type and action handler list.
    
    The notification should be included if the IMAN_Smtp_Mail_Notify handler is used.
    
    
    :var target: The object to monitor for the specified event type.  This can also be Business Object type for class
    based ImanSubscription.
    :var subscriber: The User that is subscribing to the event.
    :var eventType: Type of event that will trigger the subscription.
    :var handlers: An ordered list of handlers to execute when processing the Subscription.  Handlers will be executed
    in the order they appear in this list.  Duplicates are not allowed.  Each handler specified will also have its own
    set of handler parameters that will be passed to the handler during its execution.
    :var condition: The Condition to evaluate when processing this Subscription.  Only valid for class based
    Subscriptions.  Can be NULL.
    :var criteria: Attribute criteria to process when the Subscription is being executed.  If these attribute values
    match those of the object, the Subscription will apply.  Each entry in the list will be compared to the next using
    the logicOperator of the next value.  The logicOperator of the first entry will not be used.  Can be empty.
    :var revisionOption: Indicates the type of Item Revision objects that will apply to the Subscription.
    :var executionTime: The time of day when the Subscription should be processed.  Setting NULLDATE will cause the
    handlers to be run as soon as resources are available.
    :var expirationDate: The date when the Subscription expires and will no longer be processed.  Setting NULLDATE will
    cause the Subscription to never expire.
    :var notification: Specifies the parameters to be used when sending the notification e-mail.  Should be present for
    notification handlers.  Can be NULL for all non-notification handlers.
    """
    target: BusinessObject = None
    subscriber: User = None
    eventType: ImanEventType = None
    handlers: List[ActionHandler] = ()
    condition: Condition = None
    criteria: List[AttributeCriteria] = ()
    revisionOption: RevisionOptions = None
    executionTime: datetime = None
    expirationDate: datetime = None
    notification: NotificationDefinition = None


@dataclass
class SubscriptionsResponse(TcBaseObj):
    """
    List of the ImanSubscription objects from the operation.
    
    :var subscriptions: List of the Subscription objects from the operation.
    :var serviceData: Returned service data.
    """
    subscriptions: List[ImanSubscription] = ()
    serviceData: ServiceData = None


@dataclass
class ActionHandler(TcBaseObj):
    """
    Each handler will have an ImanActionHandler that describes the function to execute, and its own set of parameters. 
    These parameters are made up as a list of strings.  Not all handlers require parameters, but they should be present
    in the correct format if the handler expects them.  There will be no validation of the parameters when the
    Subscription is being created or modified.
    In the case of the default notification handler, the parameters should contain a list of the users to receive the
    e-mail.
    
    :var handler: The ImanActionHandler functionality to execute when the Subscription is processed.
    :var parameters: List of values to be passed to the handler during execution.
    """
    handler: ImanActionHandler = None
    parameters: List[str] = ()


class LogicOperators(Enum):
    """
    This enumeration lists the logic operators available when comparing multiple attribute comparisons as part of an
    attribute criteria on a Subscription.
    """
    LogicalAnd = 'LogicalAnd'
    LogicalOr = 'LogicalOr'


class MathOperators(Enum):
    """
    This enumeration lists the math operators available when comparing an attribute value to a specified string as part
    of an attribute criteria on a Subscription.
    """
    EqualTo = 'EqualTo'
    NotEqualTo = 'NotEqualTo'
    GreaterThan = 'GreaterThan'
    LessThan = 'LessThan'
    GreaterThanEqualTo = 'GreaterThanEqualTo'
    LessThanEqualTo = 'LessThanEqualTo'


class RevisionOptions(Enum):
    """
    Specifies which level of Item Revisions should be considered for Item  based Subscriptions.  This only applies to
    Assign Status, Attach and New Item Rev events on target objects of type Item.  For example, if a New Item Rev
    Subscription is defined with the BaselineRevisions option, only new Baseline Item Revisions will trigger the
    Subscription.
    
    NoRevisions is the default value for events other that those listed above.
    """
    NoRevisions = 'NoRevisions'
    AllRevisions = 'AllRevisions'
    ItemRevisions = 'ItemRevisions'
    BaselineRevisions = 'BaselineRevisions'



from __future__ import annotations

from typing import List
from tcsoa.gen.ProjectManagementAw._2020_05.ScheduleManagementAw import ResourceChartResponse, NameValueStringPair
from tcsoa.gen.BusinessObjects import Schedule, WorkspaceObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from datetime import datetime


class ScheduleManagementAwService(TcService):

    @classmethod
    def shiftSchedules(cls, scheduleUIDs: List[str], shiftByDays: int, shiftByDate: datetime, runInBackground: bool) -> ServiceData:
        """
        Shifts the given list of Schedule objects to a new date based on the given number of days or date. In case of
        background mode, this operation files an asynchronous request to shift the Schedule and releases the client
        immediately so that the user can perform other operation.
        """
        return cls.execute_soa_method(
            method_name='shiftSchedules',
            library='ProjectManagementAw',
            service_date='2020_05',
            service_name='ScheduleManagementAw',
            params={'scheduleUIDs': scheduleUIDs, 'shiftByDays': shiftByDays, 'shiftByDate': shiftByDate, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def shiftSchedulesAsync(cls, scheduleUIDs: List[str], shiftByDays: int, shiftByDate: datetime) -> None:
        """
        Shifts the given list of Schedule objects to a new date based on the given number of days or date. This
        operation runs asynchronously in its own server in the background.
        """
        return cls.execute_soa_method(
            method_name='shiftSchedulesAsync',
            library='ProjectManagementAw',
            service_date='2020_05',
            service_name='ScheduleManagementAw',
            params={'scheduleUIDs': scheduleUIDs, 'shiftByDays': shiftByDays, 'shiftByDate': shiftByDate},
            response_cls=None,
        )

    @classmethod
    def getResourceChartInfo(cls, resources: List[str], assignedObjects: List[WorkspaceObject], startDate: datetime, endDate: datetime, schedulesToInclude: List[Schedule], loadOptions: List[NameValueStringPair]) -> ResourceChartResponse:
        """
        The operation returns the resource chart information for the given resources assigned to the context objects.
        The resource load will be calculated for the date range between the given start and finish dates. The
        schedulesToInclude specifies the list of Schedule objects to be considered for calculating the resource chart
        information.
        """
        return cls.execute_soa_method(
            method_name='getResourceChartInfo',
            library='ProjectManagementAw',
            service_date='2020_05',
            service_name='ScheduleManagementAw',
            params={'resources': resources, 'assignedObjects': assignedObjects, 'startDate': startDate, 'endDate': endDate, 'schedulesToInclude': schedulesToInclude, 'loadOptions': loadOptions},
            response_cls=ResourceChartResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Internal.Core._2009_10.Thumbnail import ThumbnailFileTicketsResponse, UpdateThumbnailInputs, SearchOrders
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ThumbnailService(TcService):

    @classmethod
    def getThumbnailFileTickets(cls, businessObjects: List[BusinessObject], searchOrders: SearchOrders) -> ThumbnailFileTicketsResponse:
        """
        Given a list of business objects  and relation /dataset type search orders, gets the valid thumbnail file
        tickets for the list of business objects. These file tickets can be used later to download the thumbnail images
        using File Client Cache (FCC).
        
        Use cases:
        Display thumbnails in advanced search results
        """
        return cls.execute_soa_method(
            method_name='getThumbnailFileTickets',
            library='Internal-Core',
            service_date='2009_10',
            service_name='Thumbnail',
            params={'businessObjects': businessObjects, 'searchOrders': searchOrders},
            response_cls=ThumbnailFileTicketsResponse,
        )

    @classmethod
    def updateThumbnail(cls, businessObject: List[BusinessObject], updateThumbnailInputs: List[UpdateThumbnailInputs]) -> ServiceData:
        """
        This operation is to update thumbnail file for given business objects based on given user update thumbnail
        inputs. The user can specify either which source Dataset should be used to generate a thumbnail or let the
        system decide it based on thumbnail generation preferences.
        
        Use cases:
        -     Generate thumbnails at Item  create
        -     Update thumbnails for ItemRevision
        -     Generate thumbnails when Dataset is associated
        
        """
        return cls.execute_soa_method(
            method_name='updateThumbnail',
            library='Internal-Core',
            service_date='2009_10',
            service_name='Thumbnail',
            params={'businessObject': businessObject, 'updateThumbnailInputs': updateThumbnailInputs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CutItemParam(TcBaseObj):
    """
    Structure contains the item revision to be deleted and the BOMWindow that is using it.
    
    :var parent: The BOMWindow where the selected items to be deleted appear.
    :var objs: A list of selected items to be deleted.
    """
    parent: BusinessObject = None
    objs: List[BusinessObject] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class PreferenceManagementService(TcService):

    @classmethod
    def deletePreferenceCategories(cls, categoryNames: List[str]) -> ServiceData:
        """
        Deletes preference category for each provided category name if there are no preferences assigned to the
        category.
        Only Site administrators can delete an existing category.
        
        Use cases:
        Delete an existing preference category:
        Site administrator that wants to delete an existing category "MyCategory" using the deletePreferenceCategories
        operation by providing a name (e.g. " MyCategory ") as the category name. Multiple names can be provided to
        delete multiple categories.
        """
        return cls.execute_soa_method(
            method_name='deletePreferenceCategories',
            library='Internal-Administration',
            service_date='2018_06',
            service_name='PreferenceManagement',
            params={'categoryNames': categoryNames},
            response_cls=ServiceData,
        )



from __future__ import annotations

from enum import Enum
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LoggerLevel(TcBaseObj):
    """
    Logger name and logging priority level of each of the loggers.
    
    :var loggerName: Name of the logger
    :var priority: Logging priority level of the logger
    """
    loggerName: str = ''
    priority: LogLevel = None


@dataclass
class LoggerLevelResponse(TcBaseObj):
    """
    Logger name and current log level of each of the loggers
    
    :var loggerList: List of loggers, and the corresponding log level
    """
    loggerList: List[LoggerLevel] = ()


@dataclass
class ModuleStatus(TcBaseObj):
    """
    Set the journaling status for the module with the given name
    
    :var status: true if module needs to be turned on for journaling.
    :var moduleName: Name of the journaling module
    """
    status: bool = False
    moduleName: str = ''


@dataclass
class ModuleStatusResponse(TcBaseObj):
    """
    Get module name and the current status of the module.
    
    :var moduleList: Vector of module name and status.
    """
    moduleList: List[ModuleStatus] = ()


@dataclass
class ReturnStatus(TcBaseObj):
    """
    Returns status of checking, journaling, statistics, TAO level, and SQL logging.
    
    :var checkingStatus: Checking level, returns integer, 0, 1, or 2.  The checking level is used in low level  libsyss
    framework code, to control the extent of runtime diagnostic checking.
    - 0 production mode
    - 1 checking on
    - 2 extensive checking.
    
    
    :var journalStatus: True if entries are being written to the journal file,  false otherwise.
    :var statisticsStatus: True if summary statistics are collected, false otherwise.True if summary statistics are
    collected, false otherwise.
    :var taoStatus: An integer 0 through 10 representing the TAO (CORBA server) log level.
    :var sqlLogStatus: A structure holding the various SQL Logging settings.
    """
    checkingStatus: int = 0
    journalStatus: bool = False
    statisticsStatus: bool = False
    taoStatus: int = 0
    sqlLogStatus: SQLLogInfo = None


@dataclass
class SQLLogInfo(TcBaseObj):
    """
    A structure containing SQL logging option settings.
    
    :var debugState: True if SQL debug information is written to the syslog file, false otherwise.  This must be true
    for the other SQL debug settings to take effect.
    :var showBindVariables: True if SQL bind variables and their values are shown, false otherwise.
    :var profile: True if SQL profiling information is shown, false otherwise.
    :var toJournal: True if the SQL debug information is written to the journal file as well as to the syslog.  False
    if the SQL debug information is written only to the syslog file.
    :var timing: True if SQL statement timing information is shown, false otherwise.
    """
    debugState: bool = False
    showBindVariables: bool = False
    profile: bool = False
    toJournal: bool = False
    timing: bool = False


@dataclass
class SQLLogInfoResponse(TcBaseObj):
    """
    Current status of SQL logging settings.
    
    :var sqlLogInfo: A structure containing SQL logging settings.
    """
    sqlLogInfo: SQLLogInfo = None


@dataclass
class TAOLogLevelResponse(TcBaseObj):
    """
    Structure holding the TAO log level.
    
    :var level: The current TAO log level.
    """
    level: int = 0


@dataclass
class CheckingLevelResponse(TcBaseObj):
    """
    Checking level
    
    :var level: Checking level 0, 1, or 2.
    """
    level: int = 0


class LogLevel(Enum):
    """
    Enumerated list of logger priority levels
    """
    LOG_FATAL = 'LOG_FATAL'
    LOG_ERROR = 'LOG_ERROR'
    LOG_WARN = 'LOG_WARN'
    LOG_INFO = 'LOG_INFO'
    LOG_DEBUG = 'LOG_DEBUG'
    LOG_TRACE = 'LOG_TRACE'
    LOG_DEFAULT = 'LOG_DEFAULT'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, UserSession
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetStyleSheetClassificationData(TcBaseObj):
    """
    The associated classification trace and classification object associated with the specified Business Object.
    
    :var classificationObject: The Classification object to which this object has been associated.
    :var classificationTrace: The classification trace for this classification assignment.
    """
    classificationObject: BusinessObject = None
    classificationTrace: List[str] = ()


@dataclass
class GetStyleSheetDatasetInfo(TcBaseObj):
    """
    Contains the style sheet dataset info.
    
    :var datasetLastSaveDate: Contains the last save date(lsd) of the dataset
    :var datasetContent: The content of the dataset.
    """
    datasetLastSaveDate: datetime = None
    datasetContent: str = ''


@dataclass
class GetTCSessionInfoResponse(TcBaseObj):
    """
    Data structure representing the different current user's Teamcenter session information.
    
    :var userSession: The Teamcenter user session information.
    :var serviceData: The service data.
    :var hasProjects: Indicates whether this user has projects.
    """
    userSession: UserSession = None
    serviceData: ServiceData = None
    hasProjects: bool = False


@dataclass
class GetTCSessionInfoResponse3(TcBaseObj):
    """
    Data structure representing the current user's Teamcenter session information.
    
    :var userSession: The Teamcenter user session object
    :var extraInfoOut: Map of key/value pairs (string/string). Some/all/none of the following keys and values are
    returned, depending on what was passed in extraInfoIn: - TCServerVersion : The version of the Teamcenter server. -
    hasProjects : "true" or "false" depending on whether the user has projects
    :var serviceData: The service data
    """
    userSession: UserSession = None
    extraInfoOut: ExtraInfo = None
    serviceData: ServiceData = None


"""
Map of extra information
"""
ExtraInfo = Dict[str, str]



from __future__ import annotations

from typing import List
from tcsoa.gen.Manufacturing._2013_12.ResourceManagement import RMFileTicket
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GtcPackageImportResult(TcBaseObj):
    """
    Information related to importing a product from a GTC Package.
    
    
    :var status: The resulting status of an attempt to perform a GTC package import. 
    If the status value is 0, the import step succeeded. If it is not 0, the import step failed. 
    The details why the import failed are described in the report.
    
    :var icoId: The Classification Object ID of the imported product.
    
    :var icoUid: The Unique ID of the Classification Object of the imported product.
    :var wsoUid: The Unique ID of the workspace object of the imported product.
    
    :var classId: The ID of the class of the product's classifying Classification Object.
    
    :var report: A list of reports containing information, warnings, and errors related to importing the product.
    """
    status: int = 0
    icoId: str = ''
    icoUid: str = ''
    wsoUid: str = ''
    classId: str = ''
    report: List[str] = ()


@dataclass
class ImportStep3DModels2Response(TcBaseObj):
    """
    Information related to importing 3D model data from a GTC package.
    
    :var serviceData: Service data that can contain error descriptions.
    
    :var importStep3DModelResults: Information related to importing 3D model data from a GTC Package.
    
    """
    serviceData: ServiceData = None
    importStep3DModelResults: List[GtcPackageImportResult] = ()


@dataclass
class ImportStepP21Files3Response(TcBaseObj):
    """
    Information related to importing the products from STEP Part 21 files.
    
    :var serviceData: Service data that can contain error descriptions.
    
    :var logFileTicket: The File Management Service's (FMS) ticket for the STEP Part 21 import log file.
    
    :var importStepP21ProductResults: A list of information related to importing the products from STEP Part 21 files.
    """
    serviceData: ServiceData = None
    logFileTicket: RMFileTicket = None
    importStepP21ProductResults: List[GtcPackageImportResult] = ()


@dataclass
class MapClassificationObjectResponse(TcBaseObj):
    """
    Information related to mapping ICOs from one Classification class to another.
    
    :var serviceData: Service data that can contain error descriptions.
    
    :var targetIcoUid: The Teamcenter unique ID of the target ICO that the source ICO was mapped to.
    :var targetWsoUid: The Teamcenter unique ID of the target workspace object that the source ICO was mapped to.
    """
    serviceData: ServiceData = None
    targetIcoUid: str = ''
    targetWsoUid: str = ''


@dataclass
class UnzipGtcPackageResponse(TcBaseObj):
    """
    Information related to GTC package that was unzipped on the Teamcenter server.
    
    :var serviceData: Service data that can contain error descriptions.
    
    :var gtcPackageServerDirectory: The relative server-sided directory (based on the TC_DATA directory) where the GTC
    package was unzipped into.
    """
    serviceData: ServiceData = None
    gtcPackageServerDirectory: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Core._2018_12.Licensing import LicenseServerInput2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class LicensingService(TcService):

    @classmethod
    def updateLicenseServer2(cls, inputObjects: List[LicenseServerInput2]) -> ServiceData:
        """
        This operation modifies the 'Fnd0LicenseServer' business object for each 'LicenseServerInput2' supplied. The
        'LicenseServerInput2' structure contains information for properties such as license server name, host, port,
        type, protocol, failover servers and multiple servers for a given license server.  A license server location is
        defined by its host, port, and protocol.  It is not allowed to have two license server names pointing to the
        same license server location. The user performing the operation needs administrator privileges
        """
        return cls.execute_soa_method(
            method_name='updateLicenseServer2',
            library='Internal-Core',
            service_date='2018_12',
            service_name='Licensing',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetExportedObjectsResponse(TcBaseObj):
    """
    It holds list of objects that were exported to target site and errors if any.
    
    :var exportedObjectsList: A list of objects that were exported to target site
    :var serviceData: The service data
    """
    exportedObjectsList: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class NamesAndValues(TcBaseObj):
    """
    It holds name of session or transfer option and its value.
    
    :var name: The name of option
    :var value: The value of option
    """
    name: str = ''
    value: str = ''


@dataclass
class TransferFormula(TcBaseObj):
    """
    It has information about option set, list of options and their values that were overridden by user, session options
    and name of transfer mode, FMS ticket of xlst file that was used during transfer.
    
    :var optionSetName: Option set name
    :var optionSetUid: Option set UID [This parameter is ignored]
    :var optionOverrides: This is the list of options and values that user has overridden from the TransferOptionSet
    object specified above.
    
    :var sessionOptions: This is the list of session options and values (options which are not part of the option set )
    that user has specified.
    
    :var transferModeName: Transfer mode name
    :var transferModeUid: Transfer mode UID [This gets ignored]
    :var xsltFMSTicket:  FMS ticket of xslt file.
    :var reason: Reason for transfer
    """
    optionSetName: str = ''
    optionSetUid: str = ''
    optionOverrides: List[NamesAndValues] = ()
    sessionOptions: List[NamesAndValues] = ()
    transferModeName: str = ''
    transferModeUid: str = ''
    xsltFMSTicket: str = ''
    reason: str = ''



from __future__ import annotations

from tcsoa.gen.Classification._2011_06.Classification import HierarchyInfoAndOptions, GetLibraryHierarchyResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def deleteChildrenInHierarchy(cls, optionsInput: List[HierarchyInfoAndOptions]) -> ServiceData:
        """
        Deletes classification class hierarchy based on the classification class identifier. All the child classes can
        be recursively deleted along with any classification views associated with those classification classes. If
        needed, the classification objects associated with classification classes & any workspace objects associated
        with the classification objects can also be deleted
        
        Use cases:
        User wants to delete a classification class hierarchy, or a part of it. User may also need to delete the
        associated data for these classes such as classification views, classification objects or workspace objects
        """
        return cls.execute_soa_method(
            method_name='deleteChildrenInHierarchy',
            library='Classification',
            service_date='2011_06',
            service_name='Classification',
            params={'optionsInput': optionsInput},
            response_cls=ServiceData,
        )

    @classmethod
    def getLibraryHierarchy(cls, libraryValues: List[str]) -> GetLibraryHierarchyResponse:
        """
        Gets the classification class details such as class ID, parent information, child count etc. for the specified
        library values criteria
        
        Use cases:
        The operation is called when the user queries to get class hierarchy information using the given library
        values.  The operation is typically used for data dictionary related functionality in classification area, and
        the library components are created using this feature in Teamcenter. Data dictionary is a central
        organizational repository for reusable components.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getLibraryHierarchy',
            library='Classification',
            service_date='2011_06',
            service_name='Classification',
            params={'libraryValues': libraryValues},
            response_cls=GetLibraryHierarchyResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, VariantRule, RevisionRule, StructureContext, ApprSearchCriteriaGroup, Item, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class StructureCntxtObjectInfo(TcBaseObj):
    """
    This structure contains the information required to create or update the SCO.
    
    :var prdItemRevision: Product Item Revision used during performing the search.
    :var revRule: Revision Rule used during performing the search. For e.g, 'Latest working'.
    :var varRule: Variant Rule set while performing the search.
    :var workParts: List of work parts selected from DesignContext(DC) UserInterface(UI).
    :var tgtBomLines: List of all the Target BOMLine objects selected in DC UI.
    :var bgdBomLines: List of Background BOMLine objects returned by the search.
    :var selTgtBomLines: List of selected Target BOMLine objects used. This is used for storing     the selections.
    :var selBgdBomLines: List of selected Background BOMLine objects. This is used for storing the selections.
    :var tgtSearchCrtGrp: Object of Appearance Search Criteria Group for Target.
    :var bgdSearchCrtGrp: Object of Appearance Search Criteria Group for Background.
    """
    prdItemRevision: ItemRevision = None
    revRule: RevisionRule = None
    varRule: VariantRule = None
    workParts: List[Item] = ()
    tgtBomLines: List[BOMLine] = ()
    bgdBomLines: List[BOMLine] = ()
    selTgtBomLines: List[BOMLine] = ()
    selBgdBomLines: List[BOMLine] = ()
    tgtSearchCrtGrp: ApprSearchCriteriaGroup = None
    bgdSearchCrtGrp: ApprSearchCriteriaGroup = None


@dataclass
class UpdateSCOInputInfo(TcBaseObj):
    """
    This structure contains the StructureContext object and 'StructureCntxtObjectInfo' object containing information
    that needs to be updated.
    
    :var strCntxtObject: Structure Context Object that needs to be updated. This SCO object will be updated with the
    modified values.
    :var scoInfo: Object of 'StructureCntxtObjectInfo' structure. It internally holds all the details related to the
    search criteria and search results.
    """
    strCntxtObject: StructureContext = None
    scoInfo: StructureCntxtObjectInfo = None


@dataclass
class UpdateSCOResponse(TcBaseObj):
    """
    This structure contains the updated StructureContext object and error, if any, in serviceData object.
    
    :var strCntxtObject: Object of StructureContext updated with the values supplied through the input parameter.
    :var serviceData: Contains any exceptions if occurred during updation of SCOs.
    """
    strCntxtObject: StructureContext = None
    serviceData: ServiceData = None


@dataclass
class CreateSCOInputInfo(TcBaseObj):
    """
    This structure specifies the type, name and description information of a SCO object.
    
    :var scoType: Type of the Structure Context Object. For e.g. VisStructureContext.
    :var scoName: Name by which the StructureContext Object will be created.
    :var scoDesc: Description provided while creating the StructureContext object in DC UI.
    :var scoInfo: 'StructureCntxtObjectInfo' object which contains all the details related to the search criteria and
    search results.
    """
    scoType: str = ''
    scoName: str = ''
    scoDesc: str = ''
    scoInfo: StructureCntxtObjectInfo = None


@dataclass
class CreateSCOResponse(TcBaseObj):
    """
    This structure contains newly created StructureContext object and 'ServiceData' object
    
    :var strCntxtObject: Object of StructureContext created newly using the values passed in the     input parameter.
    This object encapsulates complete DesignContext session information.
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during creation of SCO.
    """
    strCntxtObject: StructureContext = None
    serviceData: ServiceData = None



from tcsoa.gen.Internal_Core._2018_11.services import LogicalObjectService as imp0
from tcsoa.gen.Internal_Core._2020_04.services import LogicalObjectService as imp1
from tcsoa.gen.Internal_Core._2017_11.services import LogicalObjectService as imp2
from tcsoa.gen.Internal_Core._2018_06.services import LogicalObjectService as imp3
from tcsoa.gen.Internal_Core._2020_01.services import ActiveModelerService as imp4
from tcsoa.gen.Internal_Core._2008_06.services import SessionService as imp5
from tcsoa.gen.Internal_Core._2008_06.services import FileManagementService as imp6
from tcsoa.gen.Internal_Core._2010_09.services import FileManagementService as imp7
from tcsoa.gen.Internal_Core._2008_06.services import DataManagementService as imp8
from tcsoa.gen.Internal_Core._2008_06.services import DispatcherManagementService as imp9
from tcsoa.gen.Internal_Core._2012_10.services import DataManagementService as imp10
from tcsoa.gen.Internal_Core._2008_03.services import SessionService as imp11
from tcsoa.gen.Internal_Core._2007_01.services import DataManagementService as imp12
from tcsoa.gen.Internal_Core._2010_09.services import DataManagementService as imp13
from tcsoa.gen.Internal_Core._2013_05.services import ProjectLevelSecurityService as imp14
from tcsoa.gen.Internal_Core._2007_06.services import ProjectLevelSecurityService as imp15
from tcsoa.gen.Internal_Core._2013_05.services import LicensingService as imp16
from tcsoa.gen.Internal_Core._2014_10.services import FileManagementService as imp17
from tcsoa.gen.Internal_Core._2017_05.services import FileManagementService as imp18
from tcsoa.gen.Internal_Core._2010_04.services import ProjectLevelSecurityService as imp19
from tcsoa.gen.Internal_Core._2007_12.services import SessionService as imp20
from tcsoa.gen.Internal_Core._2014_11.services import SessionService as imp21
from tcsoa.gen.Internal_Core._2013_05.services import PresentationManagementService as imp22
from tcsoa.gen.Internal_Core._2017_05.services import PresentationManagementService as imp23
from tcsoa.gen.Internal_Core._2017_11.services import TypeService as imp24
from tcsoa.gen.Internal_Core._2010_04.services import DataManagementService as imp25
from tcsoa.gen.Internal_Core._2017_11.services import DataManagementService as imp26
from tcsoa.gen.Internal_Core._2009_10.services import ThumbnailService as imp27
from tcsoa.gen.Internal_Core._2013_05.services import ThumbnailService as imp28
from tcsoa.gen.Internal_Core._2018_11.services import FileManagementService as imp29
from tcsoa.gen.Internal_Core._2012_02.services import DataManagementService as imp30
from tcsoa.gen.Internal_Core._2006_03.services import SessionService as imp31
from tcsoa.gen.Internal_Core._2011_06.services import ICTService as imp32
from tcsoa.gen.Internal_Core._2007_05.services import SessionService as imp33
from tcsoa.gen.Internal_Core._2016_10.services import DataManagementService as imp34
from tcsoa.gen.Internal_Core._2012_09.services import EnvelopeService as imp35
from tcsoa.gen.Internal_Core._2014_10.services import LicensingService as imp36
from tcsoa.gen.Internal_Core._2018_12.services import LicensingService as imp37
from tcsoa.gen.Internal_Core._2010_04.services import StructureManagementService as imp38
from tcsoa.gen.Internal_Core._2007_09.services import DataManagementService as imp39
from tcsoa.base import TcService


class LogicalObjectService(TcService):
    addIncludedLogicalObjects = imp0.addIncludedLogicalObjects
    addMemAndPresentedPropsWithWrite = imp1.addMemAndPresentedPropsWithWrite
    addMembersAndPresentedProps = imp2.addMembersAndPresentedProps
    addMembersAndPresentedProps2 = imp3.addMembersAndPresentedProps2
    createLogicalObjectTypes = imp2.createLogicalObjectTypes
    createLogicalObjectTypes2 = imp3.createLogicalObjectTypes2
    deleteLogicalObjectTypes = imp2.deleteLogicalObjectTypes
    deleteMembersAndPresentedProps = imp2.deleteMembersAndPresentedProps
    updateMembers = imp3.updateMembers


class ActiveModelerService(TcService):
    addPropertiesOnTypes = imp4.addPropertiesOnTypes
    createTypes = imp4.createTypes


class SessionService(TcService):
    cancelOperation = imp5.cancelOperation
    disableUserSessionState = imp11.disableUserSessionState
    getProperties = imp20.getProperties
    getSecurityToken = imp21.getSecurityToken
    initTypeByNames = imp31.initTypeByNames
    initTypeByUids = imp31.initTypeByUids
    refreshPOMCachePerRequestDeprecated = imp33.refreshPOMCachePerRequestDeprecated


class FileManagementService(TcService):
    commitRegularFiles = imp6.commitRegularFiles
    commitReplacedFiles = imp7.commitReplacedFiles
    getFileTransferTickets = imp6.getFileTransferTickets
    getPlmdFileTicketForDownload = imp17.getPlmdFileTicketForDownload
    getPlmdFileTicketForReplace = imp18.getPlmdFileTicketForReplace
    getPlmdFileTicketForUpload = imp17.getPlmdFileTicketForUpload
    getRegularFileTicketsForUpload = imp6.getRegularFileTicketsForUpload
    getTransientFileTicketsForDownload = imp29.getTransientFileTicketsForDownload
    getWriteTickets = imp6.getWriteTickets
    postCleanUpFileCommits = imp17.postCleanUpFileCommits
    updateImanFileCommits = imp6.updateImanFileCommits


class DataManagementService(TcService):
    createCachedRelations = imp8.createCachedRelations
    createRelateAndSubmitObjects = imp10.createRelateAndSubmitObjects
    getAttributeValues = imp12.getAttributeValues
    getDatasetFiles = imp13.getDatasetFiles
    getOrganizationInformation = imp12.getOrganizationInformation
    getSubscribableTypesAndSubTypes = imp25.getSubscribableTypesAndSubTypes
    getTCSessionAnalyticsInfo = imp26.getTCSessionAnalyticsInfo
    getViewableData = imp30.getViewableData
    multiRelationMultiLevelExpand = imp8.multiRelationMultiLevelExpand
    reviseObject = imp8.reviseObject
    reviseObjectsInBulk = imp34.reviseObjectsInBulk
    saveAsNewItemObject = imp8.saveAsNewItemObject
    saveAsObjectsInBulkAndRelate = imp34.saveAsObjectsInBulkAndRelate
    setDefaultProjectForProjectMembers = imp8.setDefaultProjectForProjectMembers
    whereUsedOccGroup = imp39.whereUsedOccGroup


class DispatcherManagementService(TcService):
    createDatasetOfVersion = imp9.createDatasetOfVersion
    insertDatasetVersion = imp9.insertDatasetVersion
    queryDispatcherRequests = imp9.queryDispatcherRequests
    updateDispatcherRequests = imp9.updateDispatcherRequests


class ProjectLevelSecurityService(TcService):
    getFilteredObjectsInProject = imp14.getFilteredObjectsInProject
    getFilteredProjectData = imp15.getFilteredProjectData
    getProjectsSmartFolderHierarchy = imp15.getProjectsSmartFolderHierarchy
    getProjectsSmartFolderHierarchy2 = imp19.getProjectsSmartFolderHierarchy2
    getTopLevelSmartFolderHierarchy = imp15.getTopLevelSmartFolderHierarchy


class LicensingService(TcService):
    getLicenseBundles = imp16.getLicenseBundles
    updateLicenseServer = imp36.updateLicenseServer
    updateLicenseServer2 = imp37.updateLicenseServer2


class PresentationManagementService(TcService):
    getSharedCommonClientFiles = imp22.getSharedCommonClientFiles
    getStylesheet = imp22.getStylesheet
    getStylesheetPerPage = imp23.getStylesheetPerPage


class TypeService(TcService):
    getSubTypeHierarchicalTrees = imp24.getSubTypeHierarchicalTrees


class ThumbnailService(TcService):
    getThumbnailFileTickets = imp27.getThumbnailFileTickets
    getThumbnailFileTickets2 = imp28.getThumbnailFileTickets2
    updateThumbnail = imp27.updateThumbnail


class ICTService(TcService):
    invokeICTMethod = imp32.invokeICTMethod


class EnvelopeService(TcService):
    sendEmail = imp35.sendEmail


class StructureManagementService(TcService):
    validateInStructureAssociations = imp38.validateInStructureAssociations



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetNamedColumnConfigsResponse(TcBaseObj):
    """
    The response structure for the getNamedColumnConfigs service operation.
    
    :var namedColumnConfigsJSON: A JSON string containing the information about the named column configuration(s).
    :var serviceData: The service data object.
    """
    namedColumnConfigsJSON: str = ''
    serviceData: ServiceData = None


@dataclass
class NamedColumnConfigInput(TcBaseObj):
    """
    Contains input information required to retrieve UI named and un-named column configurations from the Teamcenter
    database.
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property. 
    For example: The client name for Active Workspace is "AWClient".
    :var clientScopeUri: The unique name of the client scope containing column configurations. 
    
    Example: "Awp0SearchResults" is the client scope URI for search
    :var columnConfigName: The name of the column configuration.
    
    Example: "Search Results" for search location.
    :var columnConfigId: The unique identifier for the column configuration.
    Ex: "searchResultsColConfig" for search location.
    :var columnsToExclude: List of columns which should be excluded from the final list being returned. The value
    provided should be in the format "TypeName.PropertyName". Both type name and property name should be internal
    values. 
    For example: ItemRevision.sequence_id, where '.' is the delimiter.
    """
    clientName: str = ''
    clientScopeUri: str = ''
    columnConfigName: str = ''
    columnConfigId: str = ''
    columnsToExclude: List[str] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client, pixel width of the column, a flag indicating if the column should be hidden and the column
    sort order, fliters applied on a column, filterDefinition and the datatype of the property.
    
    :var displayName: The display name for the value displayed in the column header.
    :var typeName: The business object type for the value displayed in the column. This can be any valid Teamcenter
    business object type.
    :var isFilteringEnabled: If true, column filtering is enabled on this column; otherwise, column filtering is not
    supported on this column.
    :var dataType: The data type of the property for the given column.
    :var isFrozen: If true, the table has columns frozen until this column; otherwise, this column is not frozen.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: If true, the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort. Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted. Supported values are: "Ascending" and "Descending". This value will
    be empty if the column is not marked for sorting.
    :var filterValue: The filter criteria that is applied for current operation.
    :var filterDefinitionKey: The filter criteria that is applied for current operation.
    """
    displayName: str = ''
    typeName: str = ''
    isFilteringEnabled: bool = False
    dataType: str = ''
    isFrozen: bool = False
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''
    filterValue: FilterCriteria = None
    filterDefinitionKey: str = ''


@dataclass
class FilterCriteria(TcBaseObj):
    """
    A structure representing the column filter criteria.
    
    :var operation: This operation specifies the type of column filter operation applied. The supported operations are
    "CONTAINS", "EQUAL", "GREATER THAN", "LESS THAN" and "BETWEEN".
    :var values: A list of filter values saved in the database from the previous filter use case executed by the user.
    """
    operation: str = ''
    values: List[str] = ()


"""
Map from string to string.
"""
StringMap = Dict[str, str]
